import numpy as np
import json
import os
import hjson

class style():    
    RED = '\033[31m'
    GREEN = '\033[32m'
    BLUE = '\033[34m'
    RESET = '\033[0m'

# test1 = "ONE"
# test2 = "TWO"


# print(f"{style.RED}{test1} {style.BLUE}{test2}{style.RESET}")

def report_v(grid,data_input,show=True,model='urisi'):
    '''
    

    Parameters
    ----------
    grid : pydae object
        Pydae object modelling a grid.
    data_input : if dict, a dictionary with the grid parameters
                 if string, the path to the .json file containing grid parameters

    show : string, optional
        If report is print or not.
        
    model : string, optional
        Type of implemented model. The default is 'urisi'.

    Returns
    -------
    dict with the results.

    Example
    -------

    '''
    
    
    if type(data_input) == dict:
        data = data_input
        
    if type(data_input) == str:
        if os.path.splitext(data_input)[1] == '.json':
            with open(data_input,'r') as fobj:
                data = json.loads(fobj.read().replace("'",'"'))
        if os.path.splitext(data_input)[1] == '.hjson':
            with open(data_input,'r') as fobj:
                data = hjson.loads(fobj.read().replace("'",'"'))

    buses_dict = {}

    
    for bus in data['buses']:
        if not "acdc" in bus:
            bus.update({'acdc':'AC'})
        
        if bus['acdc'] == 'AC':
            if f"V_{bus['name']}_3_r" in grid.y_ini_list:
                v_n_r,v_n_i = grid.get_mvalue([f"V_{bus['name']}_3_r",f"V_{bus['name']}_3_i"])
                v_n_g = v_n_r + 1j*v_n_i
            else:
                v_n_g = 0.0
            #for ph in ['a','b','c']:
            
            ph = 0
            v_a_r,v_a_i =  grid.get_mvalue([f"V_{bus['name']}_{ph}_r",f"V_{bus['name']}_{ph}_i"])
            v_a_g = v_a_r + 1j*v_a_i
            v_a_n = v_a_g - v_n_g
            v_a_m = np.abs(v_a_n)
            v_a_a = np.rad2deg(np.angle(v_a_n))

            ph = 1
            v_b_r,v_b_i =  grid.get_mvalue([f"V_{bus['name']}_{ph}_r",f"V_{bus['name']}_{ph}_i"])
            v_b_g = v_b_r + 1j*v_b_i
            v_b_n = v_b_g - v_n_g
            v_b_m = np.abs(v_b_n)
            v_b_a = np.rad2deg(np.angle(v_b_n))

            ph = 2
            v_c_r,v_c_i =  grid.get_mvalue([f"V_{bus['name']}_{ph}_r",f"V_{bus['name']}_{ph}_i"])
            v_c_g = v_c_r + 1j*v_c_i
            v_c_n = v_c_g - v_n_g
            v_c_m = np.abs(v_c_n)
            v_c_a = np.rad2deg(np.angle(v_c_n))
    
            alpha = alpha = np.exp(2.0/3*np.pi*1j)
            v_0 =  1/3*(v_a_g+v_b_g+v_c_g)
            v_1 = 1.0/3.0*(v_a_g + v_b_g*alpha + v_c_g*alpha**2)
            v_2 = 1.0/3.0*(v_a_g + v_b_g*alpha**2 + v_c_g*alpha)
            
            # compute unbalanced as in Kersting 1ers edition pg. 266
            v_m_array = [v_a_m,v_b_m,v_c_m]
            v_m_min = np.min(v_m_array)
            v_m_max = np.max(v_m_array)
            max_dev = v_m_max - v_m_min
            v_avg = np.sum(v_m_array)/3
            unbalance = max_dev/v_avg
            
            bus[f"v_{bus['name']}_{'a'}n"] = v_a_m
            
            if show:
                print(f"V_{bus['name']}_{'a'}n: {v_a_m:7.1f}| {v_a_a:6.1f}º V,    V_{bus['name']}_{'a'}g: {np.abs(v_a_g):7.1f}| {np.angle(v_a_g,deg=True):6.1f}º V,    V_1 = {np.abs(v_1):7.1f} V, unb = {unbalance*100:3.2f}%")
                print(f"V_{bus['name']}_{'b'}n: {v_b_m:7.1f}| {v_b_a:6.1f}º V,    V_{bus['name']}_{'b'}g: {np.abs(v_b_g):7.1f}| {np.angle(v_b_g,deg=True):6.1f}º V,    V_2 = {np.abs(v_2):7.1f} V")
                print(f"V_{bus['name']}_{'c'}n: {v_c_m:7.1f}| {v_c_a:6.1f}º V,    V_{bus['name']}_{'c'}g: {np.abs(v_c_g):7.1f}| {np.angle(v_c_g,deg=True):6.1f}º V,    v_0 = {np.abs(v_0):7.1f} V")                   
                
                print(f"  V_{bus['name']}_ng: {np.abs(v_n_g):8.1f}| {np.angle(v_n_g,deg=True):8.1f}º V")
            
            buses_dict[bus['name']] = {f"v_{bus['name']}_{'a'}n":v_b_m,f"v_{bus['name']}_{'b'}n":v_b_m,f"v_{bus['name']}_{'c'}n":v_c_m}
            buses_dict[bus['name']].update({f'v_unb':unbalance,'v_ng':np.abs(v_n_g)})


        if bus['acdc'] == 'DC':

            
            ph = 0
            v_pos_r,v_neg_r =  grid.get_mvalue([f"V_{bus['name']}_0_r",f"V_{bus['name']}_1_r"])
            v_dc = v_pos_r - v_neg_r
    
            
            if show:
                print(f"V_{bus['name']}_dc: {v_dc:7.1f} V, V_{bus['name']}_pos_g: {v_pos_r:7.1f} V, V_{bus['name']}_neg_g: {v_neg_r:7.1f} V")

            # buses_dict[bus['name']] = {f"v_{bus['name']}_{'a'}n":v_b_m,f"v_{bus['name']}_{'b'}n":v_b_m,f"v_{bus['name']}_{'c'}n":v_c_m}
            # buses_dict[bus['name']].update({f'v_unb':unbalance,'v_ng':np.abs(v_n_g)})

    return buses_dict  


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym
from pydae.bmapu.sources.genape_inf import genape_inf
from pydae.bmapu.sources.vsource import vsource

def add_sources(grid):

    buses = grid.data['buses']
    buses_list = [bus['name'] for bus in buses]

    for item in grid.data['sources']:

        if item['type'] == 'genape':
            data_dict = item
            
            bus_name = item['bus']
            
            if 'name' in item:
                name = item['name']
            else:
                name = bus_name
                
            for gen_id in range(100):
                if name not in grid.generators_id_list:
                    grid.generators_id_list += [name]
                    break
                else:
                    name = name + f'_{gen_id}'
                    
            item['name'] = name
                                

            p_W, q_var = genape_inf(grid,name,bus_name,data_dict)

            # grid power injection
            idx_bus = buses_list.index(bus_name) # get the number of the bus where the syn is connected
            if not 'idx_powers' in buses[idx_bus]: buses[idx_bus].update({'idx_powers':0})
            buses[idx_bus]['idx_powers'] += 1

            S_base = sym.Symbol('S_base', real = True)
            grid.dae['g'][idx_bus*2]   += -p_W/S_base
            grid.dae['g'][idx_bus*2+1] += -q_var/S_base


        if item['type'] == 'vsource':
            data_dict = item
            
            bus_name = item['bus']
            
            if 'name' in item:
                name = item['name']
            else:
                name = bus_name
                
            for gen_id in range(100):
                if name not in grid.generators_id_list:
                    grid.generators_id_list += [name]
                    break
                else:
                    name = name + f'_{gen_id}'
                    
            item['name'] = name
                                

            vsource(grid,name,bus_name,data_dict)



        


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np

def change_line(model,bus_j,bus_k, *args,**kwagrs):
    line = kwagrs
    S_base = model.get_value('S_base')
    
    line_name = f"{bus_j}_{bus_k}"
    if 'X_pu' in line:
        if 'S_mva' in line: S_line = 1e6*line['S_mva']
        R = line['R_pu']*S_base/S_line  # in pu of the model base
        X = line['X_pu']*S_base/S_line  # in pu of the model base
    if 'X' in line:
        U_base = model.get_value(f'U_{bus_j}_n') 
        Z_base = U_base**2/S_base
        R = line['R']/Z_base  # in pu of the model base
        X = line['X']/Z_base  # in pu of the model base
    if 'X_km' in line:
        U_base = model.get_value(f'U_{bus_j}_n')
        Z_base = U_base**2/S_base
        R = line['R_km']*line['km']/Z_base  # in pu of the model base
        X = line['X_km']*line['km']/Z_base  # in pu of the model base
    if 'Bs_km' in line:
        U_base = model.get_value(f'U_{bus_j}_n')
        Z_base = U_base**2/S_base
        Y_base = 1.0/Z_base
        Bs = line['Bs_km']*line['km']/Y_base  # in pu of the model base
        bs = Bs
        model.set_value(f'bs_{line_name}',bs)

    G =  R/(R**2+X**2)
    B = -X/(R**2+X**2)
    model.set_value(f"g_{line_name}",G)
    model.set_value(f"b_{line_name}",B)

def get_line_current(model,bus_j,bus_k, units='A'):
    V_j_m,theta_j = model.get_mvalue([f'V_{bus_j}',f'theta_{bus_j}'])
    V_k_m,theta_k = model.get_mvalue([f'V_{bus_k}',f'theta_{bus_k}'])
    name = f'b_{bus_j}_{bus_k}'
    if name in model.params_list:
        B_j_k,G_j_k = model.get_mvalue([f'b_{bus_j}_{bus_k}',f'g_{bus_j}_{bus_k}'])
        Bs_j_k = model.get_value(f'bs_{bus_j}_{bus_k}')
    elif f'b_{bus_k}_{bus_j}' in model.params_list:
        B_j_k,G_j_k = model.get_mvalue([f'b_{bus_k}_{bus_j}',f'g_{bus_k}_{bus_j}'])
        Bs_j_k = model.get_value(f'bs_{bus_k}_{bus_j}')

    
    Y_j_k = G_j_k + 1j*B_j_k 
    V_j = V_j_m*np.exp(1j*theta_j)
    V_k = V_k_m*np.exp(1j*theta_k)
    I_j_k_pu = (V_j - V_k)*Y_j_k + 1j*V_j*Bs_j_k/2

    if units == 'A':
        S_b = model.get_value('S_base')
        U_b = model.get_value(f'U_{bus_j}_n')
        I_b = S_b/(np.sqrt(3)*U_b)
        I_j_k = I_j_k_pu*I_b

    return I_j_k


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def milano2ord(grid,name,bus_name,data_dict):
    r"""
 
    "syns":[
      {"bus":"1","type":"milano2","S_n":100e6,
         "X1d":0.30,     
         "X1q":0.55,  
         "R_a":0.01,
         "H":5.0,"D":1.0,
         "Omega_b":314.1592653589793,"omega_s":1.0,"K_sec":0.0,
         "K_delta":0.00}],
         
    **Auxiliar equations**

    .. math::
        :nowrap:
        
        \begin{eqnarray}
            v_d &=& V \sin(\delta - \theta) \\
            v_q &=& V \cos(\delta - \theta) \\
            p_e &=& i_d \left(v_d + R_a i_d\right) + i_q \left(v_q + R_a i_q\right)  \\   
            \omega_s &=& \omega_{coi}
        \end{eqnarray} 


    **Dynamic equations**

    .. math::
        :nowrap:
        
        \begin{eqnarray}
            \frac{ d\delta}{dt} &=& \Omega_b \left(\omega - \omega_s \right) - K_{\delta} \delta  \\
            \frac{ d\omega}{dt} &=& \frac{1}{2H} \left(p_m - p_e - D  \left(\omega - \omega_s \right) \right)  \\
        \end{eqnarray} 


    **Algebraic equations**
    

    .. math::
        :nowrap:
        
        \begin{eqnarray}
             0  &=& v_q + R_a i_q + X'_d i_d - e'_q \\
             0  &=& v_d + R_a i_d - X'_q i_q   \\
             0  &=& i_d v_d + i_q v_q - p_g  \\
             0  &=& i_d v_q - i_q v_d - q_g 
        \end{eqnarray} 

    .. table:: Constants
        :widths: auto

        ================== =========== ============================================= =========== 
        Variable           Code        Description                                   Units
        ================== =========== ============================================= ===========
        :math:`S_n`        ``S_n``     Nominal power                                  VA
        :math:`H`          ``H``       Inertia constaant                              s
        :math:`S_n`        ``S_n``     Nominal power                                  VA
        :math:`D`          ``D``       Damping coefficient                            s
        :math:`X_q`        ``X_q``     q-axis synchronous reactance                   pu-m
        :math:`X'_q`       ``X1q``     q-axis transient reactance                     pu-m
        :math:`X_d`        ``X_d``     d-axis synchronous reactance                   pu-m  
        :math:`X'_d`       ``X1d``     d-axis transient reactance                     pu-m
        :math:`R_a`        ``R_a``     Armature resistance                            pu-m
        :math:`K_{\delta}`  ``K_delta`` Reference machine constant                     -
        :math:`K_{sec}`    ``K_sec``   Secondary frequency control participation      -
        ================== =========== ============================================= ===========

    .. table:: Dynamic states
        :widths: auto

        ================= =========== ============================================= =========== 
        Variable          Code        Description                                   Units
        ================= =========== ============================================= ===========
        :math:`\delta`    ``delta``    Rotor angle                                  rad
        :math:`\omega`    ``omega``    Rotor speed                                  pu
        ================= =========== ============================================= ===========


    .. table:: Algebraic states
        :widths: auto

        ================= =========== ============================================= =========== 
        Variable          Code        Description                                   Units
        ================= =========== ============================================= ===========
        :math:`i_d`       ``i_d``      d-axis current                               pu-m
        :math:`i_q`       ``i_q``      q-axis current                               pu-m
        :math:`p_g`       ``p_g``      Active power                                 pu-m
        :math:`q_g`       ``q_g``      Reactive power                               pu-m
        ================= =========== ============================================= ===========

    .. table:: Inputs
        :widths: auto

        ================= =========== ============================================= =========== 
        Variable          Code        Description                                   Units
        ================= =========== ============================================= ===========
        :math:`e'_q`      ``e1q``      q-axis transient voltage                     pu
        :math:`p_m`       ``p_m``      Mechanical power                             pu-m
        ================= =========== ============================================= ===========

    """

    sin = sym.sin
    cos = sym.cos  

    # inputs
    V = sym.Symbol(f"V_{bus_name}", real=True)
    theta = sym.Symbol(f"theta_{bus_name}", real=True)
    p_m = sym.Symbol(f"p_m_{name}", real=True)
    e1q = sym.Symbol(f"e1q_{name}", real=True)
    omega_coi = sym.Symbol("omega_coi", real=True)   
        
    # dynamic states
    delta = sym.Symbol(f"delta_{name}", real=True)
    omega = sym.Symbol(f"omega_{name}", real=True)
    e1q = sym.Symbol(f"e1q_{name}", real=True)

    # algebraic states
    i_d = sym.Symbol(f"i_d_{name}", real=True)
    i_q = sym.Symbol(f"i_q_{name}", real=True)            
    p_g = sym.Symbol(f"p_g_{name}", real=True)
    q_g = sym.Symbol(f"q_g_{name}", real=True)

    # parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    Omega_b = sym.Symbol(f"Omega_b_{name}", real=True)            
    H = sym.Symbol(f"H_{name}", real=True)
    X1d = sym.Symbol(f"X1d_{name}", real=True)
    X1q = sym.Symbol(f"X1q_{name}", real=True)
    D = sym.Symbol(f"D_{name}", real=True)
    R_a = sym.Symbol(f"R_a_{name}", real=True)
    K_delta = sym.Symbol(f"K_delta_{name}", real=True)
    params_list = ['S_n','Omega_b','H','X1d','X1q','D','R_a','K_delta','K_sec']
    
    # auxiliar
    v_d = V*sin(delta - theta) 
    v_q = V*cos(delta - theta) 
    p_e = i_d*(v_d + R_a*i_d) + i_q*(v_q + R_a*i_q)     
    omega_s = omega_coi
                
    # dynamic equations            
    ddelta = Omega_b*(omega - omega_s) - K_delta*delta
    domega = 1/(2*H)*(p_m - p_e - D*(omega - omega_s))

    # algebraic equations   
    g_i_d  = v_q + R_a*i_q + X1d*i_d - e1q
    g_i_q  = v_d + R_a*i_d - X1q*i_q 
    g_p_g  = i_d*v_d + i_q*v_q - p_g  
    g_q_g  = i_d*v_q - i_q*v_d - q_g 
    
    # dae 
    f_syn = [ddelta,domega]
    x_syn = [ delta, omega]
    g_syn = [g_i_d,g_i_q,g_p_g,g_q_g]
    y_syn = [  i_d,  i_q,  p_g,  q_g]
    
    grid.H_total += H
    grid.omega_coi_numerator += omega*H*S_n
    grid.omega_coi_denominator += H*S_n

    grid.dae['f'] += f_syn
    grid.dae['x'] += x_syn
    grid.dae['g'] += g_syn
    grid.dae['y_ini'] += y_syn  
    grid.dae['y_run'] += y_syn  
    
    if 'v_f' in data_dict:
        grid.dae['u_ini_dict'].update({f'{e1q}':{data_dict['e1q']}})
        grid.dae['u_run_dict'].update({f'{e1q}':{data_dict['e1q']}})
    else:
        grid.dae['u_ini_dict'].update({f'{e1q}':1.0})
        grid.dae['u_run_dict'].update({f'{e1q}':1.0})

    if 'p_m' in data_dict:
        grid.dae['u_ini_dict'].update({f'{p_m}':{data_dict['p_m']}})
        grid.dae['u_run_dict'].update({f'{p_m}':{data_dict['p_m']}})
    else:
        grid.dae['u_ini_dict'].update({f'{p_m}':1.0})
        grid.dae['u_run_dict'].update({f'{p_m}':1.0})

    grid.dae['xy_0_dict'].update({str(omega):1.0})
    
    # outputs
    grid.dae['h_dict'].update({f"p_e_{name}":p_e})
    
    for item in params_list:       
        grid.dae['params_dict'].update({f"{item}_{name}":data_dict[item]})
    
    # if 'avr' in syn_data:
    #     add_avr(grid.dae,syn_data)
    #     grid.dae['u_ini_dict'].pop(str(v_f))
    #     grid.dae['u_run_dict'].pop(str(v_f))
    #     grid.dae['xy_0_dict'].update({str(v_f):1.5})

    # if 'gov' in syn_data:
    #     add_gov(grid.dae,syn_data)  
    #     grid.dae['u_ini_dict'].pop(str(p_m))
    #     grid.dae['u_run_dict'].pop(str(p_m))
    #     grid.dae['xy_0_dict'].update({str(p_m):0.5})

    # if 'pss' in syn_data:
    #     add_pss(grid.dae,syn_data)  

    p_W   = p_g * S_n
    q_var = q_g * S_n

    return p_W,q_var


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym



def load_dc(grid,data):

    buses = grid.data['buses']

    self = grid

    name = data['bus']
    v_p,v_n = sym.symbols(f'V_{name}_0_r,V_{name}_1_r', real=True)

    i_p,i_n = sym.symbols(f'i_load_{name}_p_r,i_load_{name}_n_r', real=True)
    p = sym.Symbol(f'p_load_{name}', real=True)

    
    self.dae['g'] += [i_p*(v_p - v_n) - p]

    self.dae['y_ini'] += [i_p]
    self.dae['y_run'] += [i_p]

    i_n = -i_p

    idx_r,idx_i = self.node2idx(name,'a')
    self.dae['g'] [idx_r] += i_p
    self.dae['g'] [idx_i] += 0.0

    idx_r,idx_i = self.node2idx(name,'b')
    self.dae['g'] [idx_r] += i_n
    self.dae['g'] [idx_i] += 0.0

    self.dae['u_ini_dict'].update({f'p_load_{name}':data['kW']*1e3})
    self.dae['u_run_dict'].update({f'p_load_{name}':data['kW']*1e3}) 


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def vsc_lcl_uc(grid,name,bus_name,data_dict):
    '''

    parameters
    ----------

    S_n: nominal power in VA
    F_n: nominal frequency in Hz
    X_v: coupling reactance in pu (base machine S_n)
    R_v: coupling resistance in pu (base machine S_n)
    K_delta: if K_delta>0.0 current generator is converted to reference machine 
    K_alpha: alpha gain to obtain Domega integral 

    inputs
    ------

    alpha: RoCoF in pu if K_alpha = 1.0
    omega_ref: frequency in pu
    v_ref: internal voltage reference

    example
    -------

    "vscs": [{"type":"vsc_l","S_n":1e6,"F_n":50.0,"X_s":0.1,"R_s":0.01}]

    v_t = m*v_dc/sqrt(6)

    v_t_pu*v_b_ac = m*v_dc_pu*v_b_dc/sqrt(6)

    v_dc_b = sqrt(2)*v_ac_b
    v_t_pu*v_b_ac = m*v_dc_pu*sqrt(2)*v_ac_b/sqrt(6)
    v_t_pu = m*v_dc_pu*sqrt(2)/sqrt(6) = m*v_dc_pu*sqrt(3)
    
    
    '''

    sin = sym.sin
    cos = sym.cos

    # inputs
    V_s = sym.Symbol(f"V_{bus_name}", real=True)
    theta_s = sym.Symbol(f"theta_{bus_name}", real=True)
    m = sym.Symbol(f"m_{name}", real=True)
    theta_t = sym.Symbol(f"theta_t_{name}", real=True)
    v_dc = sym.Symbol(f"v_dc_{name}", real=True)
    p_g = sym.Symbol(f"p_g_{name}", real=True)
      
    # dynamic states
    m_f = sym.Symbol(f"m_f_{name}", real=True)
    e_u = sym.Symbol(f"e_u_{name}", real=True)

    # algebraic states
    i_si = sym.Symbol(f"i_si_{name}", real=True)
    i_sr = sym.Symbol(f"i_sr_{name}", real=True)       
    p_s = sym.Symbol(f"p_s_{name}", real=True)
    q_s = sym.Symbol(f"q_s_{name}", real=True)
    p_dc = sym.Symbol(f"p_dc_{name}", real=True)
    i_u = sym.Symbol(f"i_u_{name}", real=True)
    v_u = sym.Symbol(f"v_u_{name}", real=True)

    # parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    F_n = sym.Symbol(f"F_n_{name}", real=True)            
    X_s = sym.Symbol(f"X_s_{name}", real=True)
    R_s = sym.Symbol(f"R_s_{name}", real=True)
    A_l = sym.Symbol(f"A_l_{name}", real=True)
    B_l = sym.Symbol(f"B_l_{name}", real=True)
    C_l = sym.Symbol(f"C_l_{name}", real=True)
    C_u = sym.Symbol(f"C_u_{name}", real=True)
    R_u = sym.Symbol(f"R_u_{name}", real=True)

    params_list = ['S_n','F_n','X_s','R_s','C_u','R_u']
    
    # auxiliar
    v_sr =  V_s*cos(theta_s)  # v_Q
    v_si =  V_s*sin(theta_s)  # v_D, e^(-j)
    Omega_b = 2*np.pi*F_n
    v_t_m = m*v_dc
    v_tr =  v_t_m*cos(theta_t)
    v_ti =  v_t_m*sin(theta_t)
    i_m = sym.sqrt(i_sr**2 + i_si**2)
    p_loss = A_l + B_l*i_m + C_l*i_m**2
    p_ac = p_s + R_s*i_m**2
    i_d = p_dc/v_dc
    


    # dynamic equations   
    de_u  = 1/C_u*(i_u)         

    # algebraic equations   
    g_i_si = v_ti - R_s*i_si - X_s*i_sr - v_si  
    g_i_sr = v_tr - R_s*i_sr + X_s*i_si - v_sr 
    g_p_s  =  i_sr*v_sr + i_si*v_si - p_s  
    g_q_s  =  i_sr*v_si - i_si*v_sr - q_s 
    g_p_dc = -p_dc + p_ac + p_loss
    g_i_u  = p_g - v_u*i_u - p_dc      # i_u and v_u refered to the ucap side
    g_v_u =  -v_u + e_u + (i_u)*R_u 


    # dae 
    f_vsg = [(m - m_f),de_u]
    x_vsg = [m_f,e_u]
    g_vsg = [g_i_si,g_i_sr,g_p_s,g_q_s,g_p_dc,g_i_u,g_v_u]
    y_vsg = [  i_si,  i_sr,  p_s,  q_s,  p_dc,  i_u,  v_u]

    grid.dae['f'] += f_vsg
    grid.dae['x'] += x_vsg
    grid.dae['g'] += g_vsg
    grid.dae['y_ini'] += y_vsg  
    grid.dae['y_run'] += y_vsg  
    
    grid.dae['u_ini_dict'].update({f'{m}':1.0})
    grid.dae['u_run_dict'].update({f'{m}':1.0})

    grid.dae['u_ini_dict'].update({f'{theta_t}':0.0})
    grid.dae['u_run_dict'].update({f'{theta_t}':0.0})

    grid.dae['u_ini_dict'].update({f'{v_dc}':1.2})
    grid.dae['u_run_dict'].update({f'{v_dc}':1.2})

    grid.dae['u_ini_dict'].update({f'{p_g}':0.1})
    grid.dae['u_run_dict'].update({f'{p_g}':0.1})

    p_s_0 = 0.5
    grid.dae['xy_0_dict'].update({str(p_s):p_s_0})
    grid.dae['xy_0_dict'].update({str(i_sr):0.1})

    grid.dae['xy_0_dict'].update({str(p_dc):0.0001})
    grid.dae['xy_0_dict'].update({str(v_u):0.5})

    # outputs
    grid.dae['h_dict'].update({f"{str(p_s)}":p_s})
    grid.dae['h_dict'].update({f"{str(q_s)}":q_s}) 
    grid.dae['h_dict'].update({f"{str(i_si)}":i_si})
    grid.dae['h_dict'].update({f"{str(i_sr)}":i_sr})
    grid.dae['h_dict'].update({f"{str(m_f)}":m_f})
    grid.dae['h_dict'].update({f"p_ac_{name}":p_ac})
    grid.dae['h_dict'].update({f"p_dc_{name}":p_s_0})
    grid.dae['h_dict'].update({f"i_d_{name}":i_d})
    grid.dae['h_dict'].update({f"p_g_{name}":p_g})
    grid.dae['h_dict'].update({f"p_u_{name}":i_u*v_u})

    for item in params_list:       
        grid.dae['params_dict'].update({f"{item}_{name}":data_dict[item]}) 

    if 'A_l' in data_dict:
        grid.dae['params_dict'].update({f"A_l_{name}":data_dict['A_l']}) 
        grid.dae['params_dict'].update({f"B_l_{name}":data_dict['B_l']}) 
        grid.dae['params_dict'].update({f"C_l_{name}":data_dict['C_l']}) 
    else:
        grid.dae['params_dict'].update({f"A_l_{name}":0.005}) 
        grid.dae['params_dict'].update({f"B_l_{name}":0.005}) 
        grid.dae['params_dict'].update({f"C_l_{name}":0.005})         


    p_W   = p_s * S_n
    q_var = q_s * S_n

    return p_W,q_var


 


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym
from pydae.bmapu.loads.load_zip import load_zip


def add_loads(grid):

    buses = grid.data['buses']
    buses_list = [bus['name'] for bus in buses]

    for item in grid.data['loads']:

        data_dict = item
        
        bus_name = item['bus']
        
        if 'name' in item:
            name = item['name']
        else:
            name = bus_name
            


        if item['type'] == 'zip':                    
            p_W, q_var = load_zip(grid,name,bus_name,data_dict)


        # grid power injection
        idx_bus = buses_list.index(bus_name) # get the number of the bus where the syn is connected
        if not 'idx_powers' in buses[idx_bus]: buses[idx_bus].update({'idx_powers':0})
        buses[idx_bus]['idx_powers'] += 1

        S_base = sym.Symbol('S_base', real = True)
        grid.dae['g'][idx_bus*2]   += -p_W/S_base
        grid.dae['g'][idx_bus*2+1] += -q_var/S_base


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def uvsg_mid(grid,name,bus_name,data_dict):
    """
    # auxiliar equations
    Omega_b = 2*np.pi*F_n
    omega_s = omega_coi
    v_D = V*sin(theta)  # e^(-j)
    v_Q = V*cos(theta) 
    v_d = v_D * cos(delta) - v_Q * sin(delta)   
    v_q = v_D * sin(delta) + v_Q * cos(delta)

    Domega = x_v + K_p * (p_ref - p)
    e_dv = 0.0
    epsilon_v = v_ref - V
    i_d = i_d_ref
    i_q = i_q_ref
    omega_v = Domega + 1.0
    q_ref_0 = K_p_v * epsilon_v + K_i_v * xi_v

    # dynamical equations
    ddelta   = Omega_b*(omega_v - omega_s) - K_delta*delta
    dx_v = K_i*(p_ref - p) - K_g*(omega_v - 1.0)
    de_qm = 1.0/T_q * (q - q_ref_0 - q_ref - e_qm) 
    dxi_v = epsilon_v # PI agregado

    # algebraic equations
    g_i_d_ref  = e_dv - R_v * i_d_ref - X_v * i_q_ref - v_d 
    g_i_q_ref  = e_qv - R_v * i_q_ref + X_v * i_d_ref - v_q 
    g_p  = v_d*i_d + v_q*i_q - p  
    g_q  = v_d*i_q - v_q*i_d - q 
    g_e_qv = 1.0 - e_qv - K_q*e_qm 

    {"bus":"1","type":"vsg_ll",'S_n':10e6,'F_n':50,'K_delta':0.0,
    'R_v':0.01,'X_v':0.1,'K_p':1.0,'K_i':0.1,'K_g':0.0,'K_q':20.0,
    'T_q':0.1,'K_p_v':1e-6,'K_i_v':1e-6}
    
    """

    sin = sym.sin
    cos = sym.cos
    buses = grid.data['buses']
    buses_list = [bus['name'] for bus in buses]                  

    idx_bus = buses_list.index(bus_name) # get the number of the bus where the syn is connected
    if not 'idx_powers' in buses[idx_bus]: buses[idx_bus].update({'idx_powers':0})
    buses[idx_bus]['idx_powers'] += 1

    # inputs:
    p_ref,q_ref,v_ref = sym.symbols(f'p_ref_{name},q_ref_{name},v_ref_{name}', real=True)

    # dynamical states:
    delta,x_v,e_qm,xi_v = sym.symbols(f'delta_{name},x_v_{name},e_qm_{name},xi_v_{name}', real=True)
    
    # algebraic states:
    i_d_ref,i_q_ref,p,q,e_qv = sym.symbols(f'i_d_ref_{name},i_q_ref_{name},p_{name},q_{name},e_qv_{name}', real=True)

    # params:
    S_base = sym.Symbol('S_base', real = True) # S_base = global power base, S_n = machine power base
    S_n,F_n,K_delta = sym.symbols(f'S_n_{name},F_n_{name},K_delta_{name}', real=True)
    K_p,K_i,K_g,K_i_q = sym.symbols(f'K_p_{name},K_i_{name},K_g_{name},K_i_q_{name}', real=True)
    R_v,X_v = sym.symbols(f'R_v_{name},X_v_{name}', real=True)
    K_q,T_q = sym.symbols(f'K_q_{name},T_q_{name}', real=True)
    K_p_v,K_i_v = sym.symbols(f'K_p_v_{name},K_i_v_{name}', real=True)
    params_list = ['S_n','F_n','K_delta','K_p','K_i','K_g','R_v','X_v','K_q','T_q','K_p_v','K_i_v']

    # auxiliar variables and constants
    omega_coi = sym.Symbol("omega_coi", real=True) # from global system
    V = sym.Symbol(f"V_{bus_name}", real=True) # from global system
    theta = sym.Symbol(f"theta_{bus_name}", real=True) # from global system
    i_d = sym.Symbol(f"i_d_{name}", real=True)
    i_q = sym.Symbol(f"i_q_{name}", real=True)

    # auxiliar equations
    Omega_b = 2*np.pi*F_n
    omega_s = omega_coi
    v_D = V*sin(theta)  # e^(-j)
    v_Q = V*cos(theta) 
    v_d = v_D * cos(delta) - v_Q * sin(delta)   
    v_q = v_D * sin(delta) + v_Q * cos(delta)

    Domega = x_v + K_p * (p_ref - p)
    e_dv = 0.0
    epsilon_v = v_ref - V
    i_d = i_d_ref
    i_q = i_q_ref
    omega_v = Domega + 1.0
    q_ref_0 = K_p_v * epsilon_v + K_i_v * xi_v

    # dynamical equations
    ddelta   = Omega_b*(omega_v - omega_s) - K_delta*delta
    dx_v = K_i*(p_ref - p) - K_g*(omega_v - 1.0)
    de_qm = 1.0/T_q * (q - q_ref_0 - q_ref - e_qm) 
    dxi_v = epsilon_v # PI agregado

    # algebraic equations
    g_i_d_ref  = e_dv - R_v * i_d_ref - X_v * i_q_ref - v_d 
    g_i_q_ref  = e_qv - R_v * i_q_ref + X_v * i_d_ref - v_q 
    g_p  = v_d*i_d + v_q*i_q - p  
    g_q  = v_d*i_q - v_q*i_d - q 
    g_e_qv = 1.0 - e_qv - K_q*e_qm 
    
    # DAE system update
    grid.dae['f'] += [ddelta,dx_v,de_qm,dxi_v]
    grid.dae['x'] += [ delta, x_v, e_qm, xi_v]
    grid.dae['g'] +=     [g_i_d_ref,g_i_q_ref,g_p,g_q,g_e_qv]
    grid.dae['y_ini'] += [  i_d_ref,  i_q_ref,  p,  q,  e_qv]
    grid.dae['y_run'] += [  i_d_ref,  i_q_ref,  p,  q,  e_qv]
            
    # default inputs
    grid.dae['u_ini_dict'].update({f'p_ref_{name}':0.0})
    grid.dae['u_ini_dict'].update({f'q_ref_{name}':0.0})
    grid.dae['u_ini_dict'].update({f'v_ref_{name}':1.0})
    grid.dae['u_run_dict'].update({f'p_ref_{name}':0.0})
    grid.dae['u_run_dict'].update({f'q_ref_{name}':0.0})
    grid.dae['u_run_dict'].update({f'v_ref_{name}':1.0})

    # default parameters
    for item in params_list:
        grid.dae['params_dict'].update({item + f'_{name}':data_dict[item]})

    # add speed*H term to global for COI speed computing
    H = 4.0
    grid.H_total += H
    grid.omega_coi_numerator += omega_v*H*S_n
    grid.omega_coi_denominator += H*S_n

    # DAE outputs update
    grid.dae['h_dict'].update({f"omega_v_{name}":omega_v})
    grid.dae['h_dict'].update({f"p_ref_{name}":p_ref})
    grid.dae['h_dict'].update({f"q_ref_{name}":q_ref})
    grid.dae['h_dict'].update({f"v_ref_{name}":v_ref})

    grid.dae['xy_0_dict'].update({str(e_qv):1.0}) 

    p_W   = p * S_n
    q_var = q * S_n

    return p_W,q_var



def uvsg_mid(self,vsg_data):
    sin = sym.sin
    cos = sym.cos
    buses = self.data['buses']
    buses_list = [bus['name'] for bus in buses]
                
    bus_name = vsg_data['bus']
    
    if 'name' in vsg_data:
        name = vsg_data['name']
    else:
        name = bus_name
        
    for gen_id in range(100):
        if name not in self.generators_id_list:
            self.generators_id_list += [name]
            break
        else:
            name = name + f'_{gen_id}'
            
    vsg_data['name'] = name
                        
    idx_bus = buses_list.index(bus_name) # get the number of the bus where the syn is connected
    if not 'idx_powers' in buses[idx_bus]: buses[idx_bus].update({'idx_powers':0})
    buses[idx_bus]['idx_powers'] += 1
    
    # inputs
    V = sym.Symbol(f"V_{bus_name}", real=True)
    theta = sym.Symbol(f"theta_{bus_name}", real=True)
    omega_coi = sym.Symbol("omega_coi", real=True)   
    p_g = sym.Symbol(f"p_g_{name}", real=True)
    q_s_ref = sym.Symbol(f"q_s_ref_{name}", real=True)
    e_ref = sym.Symbol(f"e_ref_{name}", real=True) 
        
    # dynamic states
    delta = sym.Symbol(f"delta_{name}", real=True)
    e_qv = sym.Symbol(f"e_qv_{name}", real=True)
    xi_p = sym.Symbol(f"xi_p_{name}", real=True)
    xi_q = sym.Symbol(f"xi_q_{name}", real=True)
    e = sym.Symbol(f"e_{name}", real=True)

    # algebraic states
    omega = sym.Symbol(f"omega_{name}", real=True)
    i_d = sym.Symbol(f"i_d_{name}", real=True)
    i_q = sym.Symbol(f"i_q_{name}", real=True)            
    p_s = sym.Symbol(f"p_s_{name}", real=True)
    q_s = sym.Symbol(f"q_s_{name}", real=True)
    p_m = sym.Symbol(f"p_m_{name}", real=True)
    p_t = sym.Symbol(f"p_t_{name}", real=True)
    p_u = sym.Symbol(f"p_u_{name}", real=True)

    # parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    Omega_b = sym.Symbol(f"Omega_b_{name}", real=True)            
    K_p = sym.Symbol(f"K_p_{name}", real=True)        
    T_p = sym.Symbol(f"T_p_{name}", real=True)
    K_q = sym.Symbol(f"K_q_{name}", real=True)        
    T_q = sym.Symbol(f"T_q_{name}", real=True)
    X_v = sym.Symbol(f"X_v_{name}", real=True)
    R_v = sym.Symbol(f"R_v_{name}", real=True)
    R_s = sym.Symbol(f"R_s_{name}", real=True)
    T_e = sym.Symbol(f"T_e_{name}", real=True)
    K_e = sym.Symbol(f"K_e_{name}", real=True)
    params_list = ['S_n','Omega_b','K_p','T_p','K_q','T_q','X_v','R_v','R_s','T_e','K_e']
    
    # auxiliar
    v_d = V*sin(delta - theta) 
    v_q = V*cos(delta - theta) 
    omega_s = omega_coi
    e_dv = 0
    epsilon_p = p_m - p_s
    epsilon_q = q_s_ref - q_s
                        
    
    # dynamic equations            
    ddelta = Omega_b*(omega - omega_s)
    dxi_p = epsilon_p
    dxi_q = epsilon_q
    de = 1/T_e*(p_g-p_t)

    # algebraic equations   
    g_omega = -omega + K_p*(epsilon_p + xi_p/T_p)
    g_e_qv = -e_qv   + K_q*(epsilon_q + xi_q/T_q)
    g_i_d  = -R_v*i_d - X_v*i_q - v_d + e_dv 
    g_i_q  = -R_v*i_q + X_v*i_d - v_q + e_qv
    g_p_s  = i_d*v_d + i_q*v_q - p_s  
    g_q_s  = i_d*v_q - i_q*v_d - q_s 
    g_p_m  = -p_m + p_g + p_u
    g_p_t  = -p_t + i_d*(v_d + R_s*i_d) + i_q*(v_q + R_s*i_q)
    g_p_u  = -p_u + K_e*(e_ref - e)
    
    
    # dae 
    f_vsg = [ddelta,dxi_p,dxi_q,de]
    x_vsg = [ delta, xi_p, xi_q, e]
    g_vsg = [g_omega,g_e_qv,g_i_d,g_i_q,g_p_s,g_q_s,g_p_m,g_p_t,g_p_u]
    y_vsg = [  omega,  e_qv,  i_d,  i_q,  p_s,  q_s,  p_m,  p_t,  p_u]
    
    # T_p = K_p*2*H
    H = T_p/(2*K_p)
    self.H_total += H
    self.omega_coi_numerator += omega*H*S_n
    self.omega_coi_denominator += H*S_n

    self.dae['f'] += f_vsg
    self.dae['x'] += x_vsg
    self.dae['g'] += g_vsg
    self.dae['y_ini'] += y_vsg  
    self.dae['y_run'] += y_vsg  
    
    if 'q_s_ref' in vsg_data:
        self.dae['u_ini_dict'].update({f'{str(q_s_ref)}':vsg_data['q_s_ref']})
        self.dae['u_run_dict'].update({f'{str(q_s_ref)}':vsg_data['q_s_ref']})
    else:
        self.dae['u_ini_dict'].update({f'{str(q_s_ref)}':0.0})
        self.dae['u_run_dict'].update({f'{str(q_s_ref)}':0.0})
        
    self.dae['u_ini_dict'].update({f'{str(p_g)}':vsg_data['p_g']})
    self.dae['u_run_dict'].update({f'{str(p_g)}':vsg_data['p_g']})

    self.dae['u_ini_dict'].update({f'{str(e_ref)}':vsg_data['e_ref']})
    self.dae['u_run_dict'].update({f'{str(e_ref)}':vsg_data['e_ref']})
    
    self.dae['xy_0_dict'].update({str(omega):1.0})
    
    # grid power injection
    S_base = sym.Symbol('S_base', real = True)
    self.dae['g'][idx_bus*2]   += -p_s*S_n/S_base
    self.dae['g'][idx_bus*2+1] += -q_s*S_n/S_base
    
    # outputs
    #self.dae['h_dict'].update({f"p_e_{name}":p_e})
    
    for item in params_list:       
        self.dae['params_dict'].update({f"{item}_{name}":vsg_data[item]})


    p_W   = p_s * S_n
    q_var = q_s * S_n

    return p_W,q_var




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# pydae documentation build configuration file, created by
# sphinx-quickstart on  .
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import os
import sys

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
import sphinx_rtd_theme

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#sys.path.insert(0, os.path.abspath('.'))
#sys.path.append('../src/')

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
needs_sphinx = '1.3'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
#    'sphinx.ext.napoleon',
    'sphinx.ext.todo',
    'sphinx.ext.intersphinx',
    'nbsphinx',
    'IPython.sphinxext.ipython_console_highlighting',
    'sphinx.ext.mathjax',  # Maths visualization
    'sphinx.ext.graphviz',  # Dependency diagrams
#    'notfound.extension',
#    'hoverxref.extension',
]

# Hoverxref Extension
hoverxref_auto_ref = True
hoverxref_mathjax = True
hoverxref_domains = ['py']


# Custom configuration
autodoc_member_order = 'bysource'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# Warning suppresses
suppress_warnings = ['image.nonlocal_uri']

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'pydae'
copyright = u'2013-2019, Juan Manuel Mauricio and the pydae development team'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '0.1'
# The full version, including alpha/beta/rc tags.
release = '0.1.1'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build', '**.ipynb_checkpoints']

# The reST default role (used for this markup: `text`) to use for all
# documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
#keep_warnings = False

#Intersphinx configuration
intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
    'numpy': ('https://docs.scipy.org/doc/numpy/', None),
    'scipy': ('https://docs.scipy.org/doc/scipy/reference', None),
    'matplotlib': ('https://matplotlib.org', None)
}

if os.environ.get('READTHEDOCS') == 'True':
    nbsphinx_execute = 'never'
else:
    nbsphinx_execute = 'always'

    # Controls when a cell will time out (defaults to 30; use -1 for no timeout):
    nbsphinx_timeout = 60

# -- Options for HTML output ----------------------------------------------

html_theme = "sphinx_rtd_theme"
#html_theme = "furo"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# HTML style
#html_style = os.path.join("css", "custom.css")

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
#html_extra_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_domain_indices = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'pydaedoc'


# -- Options for LaTeX output ---------------------------------------------

latex_engine = 'pdflatex'
latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    'papersize': 'a4paper',
    'releasename':" ",
    # Sonny, Lenny, Glenn, Conny, Rejne, Bjarne and Bjornstrup
    # 'fncychap': '\\usepackage[Lenny]{fncychap}',
    'fncychap': '\\usepackage{fncychap}',
    'fontpkg': '\\usepackage{amsmath,amsfonts,amssymb,amsthm}',

    'figure_align':'htbp',
    # The font size ('10pt', '11pt' or '12pt').
    #
    'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    'preamble': r'''
        %%%%%%%%%%%%%%%%%%%% Meher %%%%%%%%%%%%%%%%%%
        %%%add number to subsubsection 2=subsection, 3=subsubsection
        %%% below subsubsection is not good idea.
        \setcounter{secnumdepth}{3}
        %
        %%%% Table of content upto 2=subsection, 3=subsubsection
        \setcounter{tocdepth}{2}

        \usepackage{amsmath,amsfonts,amssymb,amsthm}
        \usepackage{graphicx}

        %%% reduce spaces for Table of contents, figures and tables
        %%% it is used "\addtocontents{toc}{\vskip -1.2cm}" etc. in the document
        \usepackage[notlot,nottoc,notlof]{}

        \usepackage{color}
        \usepackage{transparent}
        \usepackage{eso-pic}
        \usepackage{lipsum}

        \usepackage{footnotebackref} %%link at the footnote to go to the place of footnote in the text

        %% spacing between line
        \usepackage{setspace}
        %%%%\onehalfspacing
        %%%%\doublespacing
        \singlespacing


        %%%%%%%%%%% datetime
        \usepackage{datetime}

        \newdateformat{MonthYearFormat}{%
            \monthname[\THEMONTH], \THEYEAR}


        %% RO, LE will not work for 'oneside' layout.
        %% Change oneside to twoside in document class
        \usepackage{fancyhdr}
        \pagestyle{fancy}
        \fancyhf{}

        %%% Alternating Header for oneside
        \fancyhead[L]{\ifthenelse{\isodd{\value{page}}}{ \small \nouppercase{\leftmark} }{}}
        \fancyhead[R]{\ifthenelse{\isodd{\value{page}}}{}{ \small \nouppercase{\rightmark} }}

        %%% Alternating Header for two side
        %\fancyhead[RO]{\small \nouppercase{\rightmark}}
        %\fancyhead[LE]{\small \nouppercase{\leftmark}}

        %% for oneside: change footer at right side. If you want to use Left and right then use same as header defined above.
        \fancyfoot[R]{\ifthenelse{\isodd{\value{page}}}{{\tiny Meher Krishna Patel} }{\href{http://pythondsp.readthedocs.io/en/latest/pythondsp/toc.html}{\tiny PythonDSP}}}

        %%% Alternating Footer for two side
        %\fancyfoot[RO, RE]{\scriptsize Meher Krishna Patel (mekrip@gmail.com)}

        %%% page number
        \fancyfoot[CO, CE]{\thepage}

        \renewcommand{\headrulewidth}{0.5pt}
        \renewcommand{\footrulewidth}{0.5pt}

        \RequirePackage{tocbibind} %%% comment this to remove page number for following
        \addto\captionsenglish{\renewcommand{\contentsname}{Table of contents}}
        \addto\captionsenglish{\renewcommand{\listfigurename}{List of figures}}
        \addto\captionsenglish{\renewcommand{\listtablename}{List of tables}}
        % \addto\captionsenglish{\renewcommand{\chaptername}{Chapter}}


        %%reduce spacing for itemize
        \usepackage{enumitem}
        \setlist{nosep}

        %%%%%%%%%%% Quote Styles at the top of chapter
        \usepackage{epigraph}
        \setlength{\epigraphwidth}{0.8\columnwidth}
        \newcommand{\chapterquote}[2]{\epigraphhead[60]{\epigraph{\textit{#1}}{\textbf {\textit{--#2}}}}}
        %%%%%%%%%%% Quote for all places except Chapter
        \newcommand{\sectionquote}[2]{{\quote{\textit{``#1''}}{\textbf {\textit{--#2}}}}}
    ''',


    'maketitle': r'''
        \pagenumbering{Roman} %%% to avoid page 1 conflict with actual page 1

        \begin{titlepage}
            \centering

            \vspace*{40mm} %%% * is used to give space from top
            \textbf{\Huge {Sphinx format for Latex and HTML}}

            \vspace{0mm}
            %\begin{figure}[!h]
            %    \centering
            %    \includegraphics[scale=0.3]{logo.jpg}
            %\end{figure}

            \vspace{0mm}
            \Large \textbf{{Meher Krishna Patel}}

            \small Created on : Octorber, 2017

            \vspace*{0mm}
            \small  Last updated : \MonthYearFormat\today


            %% \vfill adds at the bottom
            \vfill
            \small \textit{More documents are freely available at }{\href{http://pythondsp.readthedocs.io/en/latest/pythondsp/toc.html}{PythonDSP}}
        \end{titlepage}

        \clearpage
        \pagenumbering{roman}
        \tableofcontents
        \listoffigures
        \listoftables
        \clearpage
        \pagenumbering{arabic}

        ''',
    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
    'sphinxsetup': \
        'hmargin={0.7in,0.7in}, vmargin={1in,1in}, \
        verbatimwithframe=true, \
        TitleColor={rgb}{0,0,0}, \
        HeaderFamily=\\rmfamily\\bfseries, \
        InnerLinkColor={rgb}{0,0,1}, \
        OuterLinkColor={rgb}{0,0,1}',

        'tableofcontents':' ',



}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
  ('index', 'pydae.tex', 'pydae Documentation',
   'Juan Manuel Mauricio', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'pydae', 'pydae Documentation',
     ['Juan Luis Cano Rodríguez'], 1)
]

# If true, show URL addresses after external links.
#man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
  ('index', 'pydae', 'pydae Documentation',
   'Juan Luis Cano Rodríguez', 'pydae', 'One line description of project.',
   'Miscellaneous'),
]

# Documents to append as an appendix to all manuals.
#texinfo_appendices = []

# If false, no module index is generated.
#texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
#texinfo_no_detailmenu = False



import numpy as np
import sympy as sym
  
    

def ac3ph4wpq(grid,vsc_data):
    '''
    Converter type p_ac,q_ac 3 phase 4 wire
    
    '''
    
    ## Model data
    bus_ac_name = vsc_data['bus_ac']
    bus_dc_name = vsc_data['bus_dc']  
    a_value  = vsc_data['a']   
    b_value  = vsc_data['b']   
    c_value  = vsc_data['c']   
    
    ## Voltages:
    v_a_r,v_a_i = sym.symbols(f'v_{bus_ac_name}_a_r,v_{bus_ac_name}_a_i',real=True)
    v_b_r,v_b_i = sym.symbols(f'v_{bus_ac_name}_b_r,v_{bus_ac_name}_b_i',real=True)
    v_c_r,v_c_i = sym.symbols(f'v_{bus_ac_name}_c_r,v_{bus_ac_name}_c_i',real=True)
    v_n_r,v_n_i = sym.symbols(f'v_{bus_ac_name}_n_r,v_{bus_ac_name}_n_i',real=True)
    v_dc_a_r,v_dc_n_r = sym.symbols(f'v_{bus_dc_name}_a_r,v_{bus_dc_name}_n_r',real=True)

    ## Currents:
    i_a_r,i_a_i = sym.symbols(f'i_vsc_{bus_ac_name}_a_r,i_vsc_{bus_ac_name}_a_i',real=True)
    i_b_r,i_b_i = sym.symbols(f'i_vsc_{bus_ac_name}_b_r,i_vsc_{bus_ac_name}_b_i',real=True)
    i_c_r,i_c_i = sym.symbols(f'i_vsc_{bus_ac_name}_c_r,i_vsc_{bus_ac_name}_c_i',real=True) 
    i_n_r,i_n_i = sym.symbols(f'i_vsc_{bus_ac_name}_n_r,i_vsc_{bus_ac_name}_n_i',real=True) 

    i_dc_a_r,i_dc_n_r = sym.symbols(f'i_vsc_{bus_dc_name}_a_r,i_vsc_{bus_dc_name}_n_r',real=True)


    
    ## Body:
        
    ### AC-side
    p_ac_a,p_ac_b,p_ac_c = sym.symbols(f'p_vsc_a_{bus_ac_name},p_vsc_b_{bus_ac_name},p_vsc_c_{bus_ac_name}',real=True)
    q_ac_a,q_ac_b,q_ac_c = sym.symbols(f'q_vsc_a_{bus_ac_name},q_vsc_b_{bus_ac_name},q_vsc_c_{bus_ac_name}',real=True)

    p_dc,p_loss = sym.symbols(f'p_vsc_{bus_dc_name},p_vsc_loss_{bus_ac_name}',real=True)
    #coef_a,coef_b,coef_c = sym.symbols(f'coef_a_{bus_ac_name},coef_b_{bus_ac_name},coef_c_{bus_ac_name}',real=True)
    
    v_a = v_a_r + 1j*v_a_i
    v_b = v_b_r + 1j*v_b_i
    v_c = v_c_r + 1j*v_c_i
    v_n = v_n_r + 1j*v_n_i

    i_a = i_a_r + 1j*i_a_i
    i_b = i_b_r + 1j*i_b_i
    i_c = i_c_r + 1j*i_c_i
    i_n = i_n_r + 1j*i_n_i

    s_a = (v_a - v_n) * sym.conjugate(i_a)
    s_b = (v_b - v_n) * sym.conjugate(i_b)
    s_c = (v_c - v_n) * sym.conjugate(i_c)

    eq_i_a_r =  sym.re(s_a) - p_ac_a
    eq_i_b_r =  sym.re(s_b) - p_ac_b
    eq_i_c_r =  sym.re(s_c) - p_ac_c
    eq_i_a_i =  sym.im(s_a) - q_ac_a
    eq_i_b_i =  sym.im(s_b) - q_ac_b
    eq_i_c_i =  sym.im(s_c) - q_ac_c
    eq_i_n_r =  sym.re(i_a+i_b+i_c+i_n)
    eq_i_n_i =  sym.im(i_a+i_b+i_c+i_n)

    i_abc_list  = [i_a_r,i_a_i,i_b_r,i_b_i,i_c_r,i_c_i,i_n_r,i_n_i]
    for itg in [1,2,3,4]:
        bus_idx = grid.nodes.index(f'{bus_ac_name}.{itg}')
        g_idx = bus_idx - grid.N_nodes_v
        grid.dae['g'][2*g_idx+0] += i_abc_list[2*(itg-1)  ]
        grid.dae['g'][2*g_idx+1] += i_abc_list[2*(itg-1)+1]

    ### DC side
    a,b,c = sym.symbols(f'a_{bus_ac_name},b_{bus_ac_name},c_{bus_ac_name}',real=True)
    i_rms_a = sym.sqrt(i_a_r**2+i_a_i**2+1e-6) 
    i_rms_b = sym.sqrt(i_b_r**2+i_b_i**2+1e-6) 
    i_rms_c = sym.sqrt(i_c_r**2+i_c_i**2+1e-6) 
    i_rms_n = sym.sqrt(i_n_r**2+i_n_i**2+1e-6) 

    p_loss_a = a + b*i_rms_a + c*i_rms_a*i_rms_a
    p_loss_b = a + b*i_rms_b + c*i_rms_b*i_rms_b
    p_loss_c = a + b*i_rms_c + c*i_rms_c*i_rms_c
    p_loss_n = a + b*i_rms_n + c*i_rms_n*i_rms_n

    p_vsc_loss = p_loss_a + p_loss_b + p_loss_c + p_loss_n
    p_ac = p_ac_a + p_ac_b + p_ac_c

    eq_p_loss = p_loss - p_vsc_loss
    eq_i_dc_a_r = i_dc_a_r + p_dc/(v_dc_a_r-v_dc_n_r+1e-8)
    eq_i_dc_n_r = i_dc_n_r + p_dc/(v_dc_n_r-v_dc_a_r+1e-8)
    eq_p_dc = -p_dc + p_ac + p_loss #- sym.Piecewise((p_loss, p_ac > 0), (-p_loss, p_ac < 0),(p_loss, True))


    ## DAE system update
    grid.dae['g'] += [eq_i_a_r,eq_i_a_i,
                      eq_i_b_r,eq_i_b_i,
                      eq_i_c_r,eq_i_c_i,
                      eq_i_n_r,eq_i_n_i]
    
    grid.dae['y'] += [   i_a_r,   i_a_i,
                         i_b_r,   i_b_i,
                         i_c_r,   i_c_i,
                         i_n_r,   i_n_i]

    grid.dae['g'] +=  [eq_i_dc_a_r,eq_i_dc_n_r,eq_p_dc,eq_p_loss]
    grid.dae['y'] +=  [   i_dc_a_r,   i_dc_n_r,   p_dc,   p_loss]  
    #grid.dae['u'].update({f'p_vsc_{bus_ac_name}':0.0,f'q_vsc_{bus_ac_name}':0.0}) 
    grid.dae['xy_0_dict'].update({f'v_{bus_dc_name}_a_r':800.0,f'v_{bus_dc_name}_n_r':0.01})
    
    #grid.dae['u'].pop(f'i_{bus_dc_name}_a_r')
    #grid.dae['u'].pop(f'i_{bus_dc_name}_n_r')
    grid.dae['params'].update({f'a_{bus_ac_name}':a_value,f'b_{bus_ac_name}':b_value,f'c_{bus_ac_name}':c_value})
    grid.dae['u'].update({f'{str(p_ac_a)}':0.0,f'{str(p_ac_b)}':0.0,f'{str(p_ac_c)}':0.0})
    grid.dae['u'].update({f'{str(q_ac_a)}':0.0,f'{str(q_ac_b)}':0.0,f'{str(q_ac_c)}':0.0})

    ## Add current injections to grid equations:
    bus_idx = grid.nodes.index(f'{bus_dc_name}.{1}')
    g_idx = bus_idx - grid.N_nodes_v
    grid.dae['g'][2*g_idx+0] += i_dc_a_r
    grid.dae['g'][2*g_idx+1] += 0.0   

    bus_idx = grid.nodes.index(f'{bus_dc_name}.{4}')
    g_idx = bus_idx - grid.N_nodes_v
    grid.dae['g'][2*g_idx+0] += i_dc_n_r
    grid.dae['g'][2*g_idx+1] += 0.0 

    grid.dae['h_dict'].update({f'p_vsc_{bus_ac_name}':sym.re(s_a)+sym.re(s_b)+sym.re(s_c)})
    grid.dae['h_dict'].update({f'p_vsc_loss_{bus_ac_name}':(p_loss_a+p_loss_b+p_loss_c+p_loss_n)})




import sympy as sym
import numpy as np

def vdc_src(grid,data):
    '''
    dc/dc converter grid former in the low voltage side
    
    '''
    
    bus_name = data['bus']
    name = bus_name
    

    #### voltages:
    v_hp, v_hn = sym.symbols(f'V_{name}_0_r,V_{name}_1_r', real=True)
    v_hp_i, v_hn_i = sym.symbols(f'V_{name}_0_i,V_{name}_1_i', real=True)
   
    ### HV-side
    e_h,v_h,i_h,p_h = sym.symbols(f'e_h_{name},v_h_{name},i_h_{name},p_h_{name}',real=True)
    R_h = sym.Symbol(f'R_h_{name}',real=True)
    v_ref = sym.Symbol(f'v_ref_{name}',real=True)
    m_h = sym.Symbol(f'm_h_{name}',real=True)
    i_g = sym.Symbol(f'i_g_{name}',real=True) 
    R_g = sym.Symbol(f'R_g_{name}',real=True)

    i_g = (v_hn + v_hp)/(2*R_g + R_h)
    v_og = i_g*R_g
    i_hp = (v_og + v_ref/2 - v_hp)/R_h
    i_hn = (v_og - v_ref/2 - v_hn)/R_h
    p_h = i_hp*v_ref

    # current injections dc LV side
    idx_hp_r,idx_hp_i = grid.node2idx(f'{name}','a')
    idx_hn_r,idx_hn_i = grid.node2idx(f'{name}','b')
    grid.dae['g'] [idx_hp_r] += -i_hp 
    grid.dae['g'] [idx_hn_r] += -i_hn
    grid.dae['g'] [idx_hp_i] += -v_hp_i/1e3
    grid.dae['g'] [idx_hn_i] += -v_hn_i/1e3

    v_ref = 800.0
    if "v_ref" in data:
        v_ref = data['v_ref']

    grid.dae['u_ini_dict'].update({f'v_ref_{name}':v_ref}) 
    grid.dae['u_run_dict'].update({f'v_ref_{name}':v_ref}) 
   
    grid.dae['params_dict'].update({f'R_h_{name}':1e-4})    
    grid.dae['params_dict'].update({f'R_g_{name}':3})   
    grid.dae['h_dict'].update({f'p_h_{name}':p_h})   

    grid.omega_coi_denominator += 1e6

def development():
    v_hp, v_hn, v_ref = sym.symbols(f'v_hp,v_hn,v_ref', real=True)
    i_hp, i_hn, i_g = sym.symbols(f'i_hp,i_hn,i_g', real=True)
    R_h, R_g = sym.symbols(f'R_h,R_g', real=True)

    v_og = i_g*R_g
    eq_i_hp = v_og + v_ref/2 - i_hp*R_h - v_hp
    eq_i_hn = v_og - v_ref/2 - i_hn*R_h - v_hn
    eq_i_g  = i_g + i_hp + i_hn

    result = sym.solve([eq_i_hp,eq_i_hn,eq_i_g],[i_hp,i_hn,i_g])
    print(result)

def test():
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db

    grid = urisi('dc_src_load.hjson')
    grid.uz_jacs = True
    grid.construct('dae_module')
    grid.compile('dae_module')

    import dae_module

    model = dae_module.model()
    model.ini({},'xy_0.json')
    model.report_y()
    model.report_z()



if __name__ == '__main__':

    #development()
    test()






import numpy as np
import sympy as sym
import pydae.build_cffi as db
import json 
import hjson
import requests

class pv_model:

    def __init__(self,name='pv'):

        self.name = name

        self.I_sc = 3.87
        self.V_oc = 42.1
        self.I_mpp = 3.56
        self.V_mpp = 33.7
        self.N_s = 72
        self.K_vt = -0.160
        self.K_it = 0.065
        self.R_pv_s = 0.5602
        self.R_pv_sh = 1862
        self.K_d = 1.3433

    def model_params(self,data_input):
        '''
        Input
        =====

        data_input: dict
            {"I_sc":3.87,"V_oc":42.1,"I_mpp":3.56,"V_mpp":33.7,"N_s":72,
            "K_vt":-0.160,"K_it":0.065,"R_s": 0.5602, "R_sh": 1862, "K_d": 1.3433}
                       
            I_sc: short circuit current # A
            I_mpp: maximum power point current # A
            V_mpp: maximum power point voltage # V
            V_oc:  open circuit voltage # V
            N_s: number of cells in series
            P_mpp: 120  # W 
            K_vt = -0.160 # Temperature Coefficient of V_oc %/Cº
            K_it = 0.065  # Temperature Coefficient of I_sc %/Cº

        Example
        =======

           http://www.posharp.com/msx120-solar-panel-from-bp-solar_p857364851d.aspx

        # from manufacturer
        I_sc = 3.87 # short circuit current
        I_mpp = 3.56 # maximum power point current
        V_mpp = 33.7 # maximum power point voltage
        V_oc = 42.1  # open circuit voltage
        N_s = 72 # number of cells in series
        P_mpp = 120 
        K_vt = -0.160 # Temperature Coefficient of V_oc %/Cº
        K_it = 0.065  # Temperature Coefficient of I_sc %/Cº	       
        '''
        from photovoltaic_modeling.parameter.parameter_extraction import ParameterExtraction

        if type(data_input) == str:
            if 'http' in data_input:
                url = data_input
                resp = requests.get(url)
                data = json.loads(resp.text)
            else:
                if os.path.splitext(data_input)[1] == '.json':
                    with open(data_input,'r') as fobj:
                        data = json.loads(fobj.read().replace("'",'"'))
                if os.path.splitext(data_input)[1] == '.hjson':
                    with open(data_input,'r') as fobj:
                        data = hjson.loads(fobj.read().replace("'",'"'))
        elif type(data_input) == dict:
            data = data_input
            
        self.data = data 

        self.I_sc = self.data['I_sc'] 
        self.V_oc = self.data['V_oc']  
        self.I_mpp= self.data['I_mpp']  
        self.V_mpp= self.data['V_mpp']  
        self.N_s = self.data['N_s']  

        # k_p = -0.5
        T_stc = 25 + 273.4

        E_c = 1.6022e-19 # Elementary charge
        Boltzmann = 1.3806e-23 # Boltzmann constant

        parameter_extraction = ParameterExtraction(self.I_sc, 
                                                   self.V_oc, 
                                                   self.I_mpp, 
                                                   self.V_mpp, 
                                                   number_of_cells_in_series = self.N_s)
        
        initial_guess = [1,1000,1] # [series_resistance_estimate,shunt_resistance_estimate,diode_quality_factor_estimate]
        parameter_extraction.calculate(initial_guess) 

        self.R_pv_s = parameter_extraction.series_resistance
        self.R_pv_sh = parameter_extraction.shunt_resistance
        self.K_d = parameter_extraction.diode_quality_factor


    def build(self):
        I_sc,I_mpp,V_mpp,V_oc = sym.symbols(f'I_sc,I_mpp,V_mpp,V_oc', real=True)
        N_s,K_vt,K_it = sym.symbols(f'N_s,K_vt,K_it', real=True)
        K_d,R_pv_s,R_pv_sh,v_mpp,i_pv_mpp = sym.symbols(f'K_d,R_pv_s,R_pv_sh,v_mpp,i_pv_mpp', real=True)

        temp_deg,irrad,i_pv,v_pv,p = sym.symbols(f'temp_deg,irrad,i_pv,v_pv,p', real=True)

        T_stc = 25 + 273.4
        E_c = 1.6022e-19 # Elementary charge
        Boltzmann = 1.3806e-23 # Boltzmann constant

        params_dict = {str(I_sc): self.I_sc,  # 3.87,
                       str(I_mpp):self.I_mpp,  # 42.1,
                       str(V_mpp):self.V_mpp, # 3.56,
                       str(V_oc): self.V_oc, # 33.7,
                       str(N_s):  self.N_s,   # 72,
                       str(K_vt): self.K_vt,  # -0.160,
                       str(K_it): self.K_it,  # 0.065
                       str(R_pv_s):  self.R_pv_s,   # 0.5602
                       str(R_pv_sh): self.R_pv_sh,  # 1862
                       str(K_d):  self.K_d}   # 1.3433


        temp_k = temp_deg + 273.4

        # V_t = K_d*Boltzmann*T_stc/E_c
        # V_oc_t = V_oc + K_vt*( temp_k - T_stc)

        I_rrad_sts = 1000
  
        # I_sc_t = I_sc*irrad/I_rrad_sts*(1 + K_it/100*(temp_k - T_stc))

        # I_0 = (I_sc_t - (V_oc_t - I_sc_t*R_pv_s)/R_pv_sh)*sym.exp(-V_oc_t/(N_s*V_t))

        # I_ph = (I_0*sym.exp(V_oc_t/(N_s*V_t)) + V_oc_t/R_pv_sh)*irrad/I_rrad_sts

        # eq_i_pv = -i_pv + I_ph - I_0 * (sym.exp((v_pv + i_pv*R_pv_s)/(N_s*V_t))-1)-(v_pv+i_pv*R_pv_s)/R_pv_sh 
        #eq_p = -p + i*v

        V_t = K_d*Boltzmann*T_stc/E_c
        V_oc_t = V_oc * (1+K_vt/100.0*( temp_k - T_stc))
        
        I_sc_t = I_sc*(1 + K_it/100*(temp_k - T_stc))
        I_0 = (I_sc_t - (V_oc_t - I_sc_t*R_pv_s)/R_pv_sh)*sym.exp(-V_oc_t/(N_s*V_t))
        I_d = I_0*(sym.exp((v_pv+i_pv*R_pv_s)/(V_t*N_s))-1)
        I_ph = I_sc_t*irrad/I_rrad_sts
        
        eq_i_pv = -i_pv + I_ph - I_d - (v_pv+i_pv*R_pv_s)/R_pv_sh 

        I_d = I_0*(sym.exp((v_mpp+i_pv_mpp*R_pv_s)/(V_t*N_s))-1)
        I_ph = I_sc_t*irrad/I_rrad_sts
        eq_i_pv_mpp  = -i_pv_mpp + I_ph - I_d - (v_mpp+i_pv_mpp*R_pv_s)/R_pv_sh

        i_mpp = I_ph - I_d - (v_mpp+i_pv_mpp*R_pv_s)/R_pv_sh
        p = i_mpp*v_mpp
        dPdv = sym.diff(p,v_mpp)
        eq_v_mpp = dPdv 

        i = sym.Symbol(f'i', real=True)

        #dp_0 = sym.diff((I_ph - I_0 * (sym.exp((v + i*R_pv_s)/(N_s*V_t))-1)-(v+i*R_pv_s)/R_pv_sh)*v,v)
        #v_mpp = sym.Symbol(f'v_mpp', real = True)
        #eq_v_mpp = dp_0.subs(v,v_mpp)

        u_ini_dict = {f'v_pv':30,f'irrad':1000,f'temp_deg':25}  # input for the initialization problem
        u_run_dict = {f'v_pv':30,f'irrad':1000,f'temp_deg':25}  # input for the running problem, its value is updated

        h_dict = {'I_ph':I_ph}
        h_dict = {'v':v_pv}
        h_dict = {'p':v_pv*i_pv}

        self.sys_dict = {'name':self.name,
                    'params_dict':params_dict,
                    'f_list':[],
                    'g_list':[eq_i_pv,eq_i_pv_mpp,eq_v_mpp],
                    'x_list':[ ],
                    'y_ini_list':[i_pv,i_pv_mpp,v_mpp],
                    'y_run_list':[i_pv,i_pv_mpp,v_mpp],
                    'u_ini_dict':u_ini_dict,
                    'u_run_dict':u_run_dict,
                    'h_dict':h_dict}

        bldr = db.builder(self.sys_dict)
        bldr.build()


if __name__ == "__main__":

    data = {"I_sc":3.87,"V_oc":42.1,"I_mpp":3.56,"V_mpp":33.7,"N_s":72,
            "K_vt":-0.160,"K_it":0.065,"R_pv_s": 0.5602, "R_pv_sh": 1862, "K_d": 1.3433}
    
    pv = pv_model('pv_test')
    pv.model_params(data)
    pv.build()

    import pv_test

    model = pv_test.model()
    model.ini({},1)
    model.report_y()




import numpy as np
import sympy as sym

def ctrl_3ph_4w_pq(grid,data,name,bus_name):


    # parameters
    S_n,U_n = sym.symbols(f'S_n_{name},U_n_{name}', real=True)
    R_s,X_s = sym.symbols(f'R_s_{name},X_s_{name}', real=True)

    ## inputs
    p_ref = sym.Symbol(f'p_ref_{name}', real=True)    
    q_ref = sym.Symbol(f'q_ref_{name}', real=True)    
    p_a_ref,q_a_ref = sym.symbols(f'p_a_ref_{name},q_a_ref_{name}', real=True)
    p_b_ref,q_b_ref = sym.symbols(f'p_b_ref_{name},q_b_ref_{name}', real=True)
    p_c_ref,q_c_ref = sym.symbols(f'p_c_ref_{name},q_c_ref_{name}', real=True)
    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'V_{bus_name}_0_r,V_{bus_name}_1_r,V_{bus_name}_2_r,V_{bus_name}_3_r,v_{bus_name}_o_r', real=True)
    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'V_{bus_name}_0_i,V_{bus_name}_1_i,V_{bus_name}_2_i,V_{bus_name}_3_i,v_{bus_name}_o_i', real=True)
    p_soc = sym.Symbol(f'p_soc_{name}',real=True)
    v_dc = sym.Symbol(f'v_dc_{name}',real=True)


    ## algebraic states
    i_sa_ref_r,i_sb_ref_r,i_sc_ref_r,i_sn_ref_r = sym.symbols(f'i_sa_ref_r_{name},i_sb_ref_r_{name},i_sc_ref_r_{name},i_sn_ref_r_{name}', real=True)
    i_sa_ref_i,i_sb_ref_i,i_sc_ref_i,i_sn_ref_i = sym.symbols(f'i_sa_ref_i_{name},i_sb_ref_i_{name},i_sc_ref_i_{name},i_sn_ref_i_{name}', real=True)
    v_ta_r,v_tb_r,v_tc_r,v_tn_r = sym.symbols(f'v_ta_r_{name},v_tb_r_{name},v_tc_r_{name},v_tn_r_{name}', real=True)
    v_ta_i,v_tb_i,v_tc_i,v_tn_i = sym.symbols(f'v_ta_i_{name},v_tb_i_{name},v_tc_i_{name},v_tn_i_{name}', real=True)


    ## dynamical states

    ## auxiliar variables
    omega = sym.Symbol(f'omega_{name}', real=True)
    p_m = sym.Symbol(f'p_m_{name}', real=True) 

    
    ## auxiliar equations
    alpha = np.exp(2.0/3*np.pi*1j)
    A_0a =  np.array([[1, 1, 1],
                    [1, alpha**2, alpha],
                    [1, alpha, alpha**2]])

    A_a0 = 1/3* np.array([[1, 1, 1],
                        [1, alpha, alpha**2],
                        [1, alpha**2, alpha]])

    omega_coi_i = 0
    HS_coi = 0

    ## POI currents
    i_sa_ref = i_sa_ref_r + 1j*i_sa_ref_i
    i_sb_ref = i_sb_ref_r + 1j*i_sb_ref_i
    i_sc_ref = i_sc_ref_r + 1j*i_sc_ref_i

    ## POI voltages
    v_sa = v_sa_r + 1j*v_sa_i
    v_sb = v_sb_r + 1j*v_sb_i
    v_sc = v_sc_r + 1j*v_sc_i

    ## VSC AC side voltages
    v_ta = v_ta_r + 1j*v_ta_i
    v_tb = v_tb_r + 1j*v_tb_i
    v_tc = v_tc_r + 1j*v_tc_i
    v_tn = v_tn_r + 1j*v_tn_i

    ## powers
    s_a_ref = (p_ref/3+p_soc/3+p_a_ref) + sym.I*(q_ref/3+q_a_ref)
    s_b_ref = (p_ref/3+p_soc/3+p_b_ref) + sym.I*(q_ref/3+q_b_ref)
    s_c_ref = (p_ref/3+p_soc/3+p_c_ref) + sym.I*(q_ref/3+q_c_ref)

    Z_s = R_s + sym.I*X_s

    ## differential equations
     
    ## algebraic equations   
    eq_i_sa_ref_cplx =  s_a_ref - v_sa*np.conjugate(i_sa_ref)
    eq_i_sb_ref_cplx =  s_b_ref - v_sb*np.conjugate(i_sb_ref)
    eq_i_sc_ref_cplx =  s_c_ref - v_sc*np.conjugate(i_sc_ref)
    eq_v_ta_cplx =  v_ta - Z_s*i_sa_ref - v_sa 
    eq_v_tb_cplx =  v_tb - Z_s*i_sb_ref - v_sb 
    eq_v_tc_cplx =  v_tc - Z_s*i_sc_ref - v_sc 
 
    grid.dae['g'] += [sym.re(eq_i_sa_ref_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_sb_ref_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_sc_ref_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_sa_ref_cplx)]
    grid.dae['g'] += [sym.im(eq_i_sb_ref_cplx)]
    grid.dae['g'] += [sym.im(eq_i_sc_ref_cplx)]

    grid.dae['g'] += [sym.re(eq_v_ta_cplx)] 
    grid.dae['g'] += [sym.re(eq_v_tb_cplx)] 
    grid.dae['g'] += [sym.re(eq_v_tc_cplx)] 
    grid.dae['g'] += [sym.im(eq_v_ta_cplx)]
    grid.dae['g'] += [sym.im(eq_v_tb_cplx)]
    grid.dae['g'] += [sym.im(eq_v_tc_cplx)]

    grid.dae['y_ini'] += [i_sa_ref_r,i_sb_ref_r,i_sc_ref_r]
    grid.dae['y_ini'] += [i_sa_ref_i,i_sb_ref_i,i_sc_ref_i]
    grid.dae['y_run'] += [i_sa_ref_r,i_sb_ref_r,i_sc_ref_r]
    grid.dae['y_run'] += [i_sa_ref_i,i_sb_ref_i,i_sc_ref_i]

    grid.dae['y_ini'] += [v_ta_r,v_tb_r,v_tc_r]
    grid.dae['y_ini'] += [v_ta_i,v_tb_i,v_tc_i]
    grid.dae['y_run'] += [v_ta_r,v_tb_r,v_tc_r]
    grid.dae['y_run'] += [v_ta_i,v_tb_i,v_tc_i]
        
    V_1 = 400/np.sqrt(3)
    V_b = V_1
    #    V_1 = 400/np.sqrt(3)*np.exp(1j*np.deg2rad(0))
    # A_1toabc = np.array([1, alpha**2, alpha])
    #V_abc = V_1 * A_1toabc 
    #e_an_r,e_bn_r,e_cn_r = V_abc.real
    #e_an_i,e_bn_i,e_cn_i = V_abc.imag

    ## inputs default values
    grid.dae['u_ini_dict'].update(
                      {f'p_a_ref_{name}':0.0,
                       f'p_b_ref_{name}':0.0,
                       f'p_c_ref_{name}':0.0,
                       f'p_soc_{name}':0.0})
    grid.dae['u_ini_dict'].update(
                      {f'q_a_ref_{name}':0.0,
                       f'q_b_ref_{name}':0.0,
                       f'q_c_ref_{name}':0.0,
                       f'q_ref_{name}':0.0})
    

    #u_dict.update({f'phi_{name}':0.0})
    grid.dae['u_run_dict'].update(
                      {f'p_a_ref_{name}':0.0,
                       f'p_b_ref_{name}':0.0,
                       f'p_c_ref_{name}':0.0,
                       f'p_soc_{name}':0.0})
    grid.dae['u_run_dict'].update(
                      {f'q_a_ref_{name}':0.0,
                       f'q_b_ref_{name}':0.0,
                       f'q_c_ref_{name}':0.0,
                       f'q_ref_{name}':0.0})

    if not p_ref in grid.dae['y_ini']:
        grid.dae['u_ini_dict'].update({f'p_ref_{name}':0.0})
        grid.dae['u_run_dict'].update({f'p_ref_{name}':0.0})


    ## parameters default values

    grid.dae['u_ini_dict'].pop(str(v_ta_r))
    grid.dae['u_ini_dict'].pop(str(v_tb_r))
    grid.dae['u_ini_dict'].pop(str(v_tc_r))
    grid.dae['u_run_dict'].pop(str(v_ta_r))
    grid.dae['u_run_dict'].pop(str(v_tb_r))
    grid.dae['u_run_dict'].pop(str(v_tc_r))
    grid.dae['u_ini_dict'].pop(str(v_ta_i))
    grid.dae['u_ini_dict'].pop(str(v_tb_i))
    grid.dae['u_ini_dict'].pop(str(v_tc_i))
    grid.dae['u_run_dict'].pop(str(v_ta_i))
    grid.dae['u_run_dict'].pop(str(v_tb_i))
    grid.dae['u_run_dict'].pop(str(v_tc_i))


    grid.dae['h_dict'].update({f'p_ref_{name}':p_ref})



def test_ib():
    '''
    test grid former connected to infinite bus
    '''
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db
    import pytest

    grid = urisi('ctrl_3ph_4w_pq_ib.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    model = temp.model()    
    p_ref = 10e3
    q_ref = 0.0
    model.ini({'p_ref_A1':p_ref,'q_ref_A1':q_ref},'xy_0.json')
    
    model.report_u()
    model.report_x()
    model.report_y()
    model.report_z()

    # S_n = model.get_value('S_n_A1')
    # assert model.get_value('omega_A1')  == pytest.approx(1)
    # assert model.get_value('omega_coi') == pytest.approx(1)
    # assert model.get_value('p_vsc_A1_a') == pytest.approx(p_c*S_n/3, rel=0.05)
    # assert model.get_value('p_vsc_A1_b') == pytest.approx(p_c*S_n/3, rel=0.05)
    # assert model.get_value('p_vsc_A1_c') == pytest.approx(p_c*S_n/3, rel=0.05)
    # assert model.get_value('p_A2') == pytest.approx(-p_c*S_n, rel=0.05)
    # assert model.get_value('q_A2') == pytest.approx(-q_ref*S_n, rel=0.05)

 

if __name__ == '__main__':

    #development()
    test_ib()


import numpy as np
import matplotlib.pyplot as plt
import ipywidgets as widgets
import pydae.svg_tools as st 
import pydae.build_cffi as db
from pydae.bmapu import bmapu_builder

plt.style.use('https://raw.githubusercontent.com/pydae/pydae/master/src/pydae/edashboards/presentation.mplstyle')
colors = plt.rcParams['axes.prop_cycle'].by_key()['color']

class dashboard():
    
    def __init__(self):

        
        self.colors = plt.rcParams['axes.prop_cycle'].by_key()['color']
        self.data = {
        "system":{"name":"wecs_pmsm_test","S_base":100e6, "K_p_agc":0.0,"K_i_agc":0.0,"K_xif":0.01},       
        "buses":[{"name":"1", "P_W":0.0,"Q_var":0.0,"U_kV":20.0},
                    {"name":"2", "P_W":0.0,"Q_var":0.0,"U_kV":20.0}
                ],
        "lines":[{"bus_j":"1", "bus_k":"2", "X_pu":0.05,"R_pu":0.01,"Bs_pu":1e-6,"S_mva":2.0}],
        "wecs":[
            {"type":"pmsm_1","bus":"1","S_n":1e6,
                "H_t":4.0,"H_r":1.0, "w_tr":5.0, "d_tr":0.01,
                "R_m":0.01,"L_m":0.05,"Phi_m":1.0,
                "R_s":0.02,"X_s":0.05,
                "K_pdc":0.1,"C_dc":0.5}],
        "genapes":[{"bus":"2","S_n":1e9,"F_n":50.0,"X_v":0.001,"R_v":0.0,"K_delta":0.001,"K_alpha":1e-6}]
        }

    def ini(self):

        import wecs_pmsm

        self.model = wecs_pmsm.model()
        self.model.Dt = 0.01
        self.model.decimation = 1
        self.model.ini({'K_pdc_1':50,"C_dc_1":0.01,"nu_w_1":10,
           "K_p_beta_1":1,"K_i_beta_1":1.0,"T_beta_1":1.0},'xy_0.json')
        
        
    def build(self):



        grid = bmapu_builder.bmapu(self.data)
        #grid.checker()
        grid.uz_jacs = True
        grid.verbose = False
        grid.build('wecs_pmsm')


    def widgets(self):
        
        model = self.model
        colors = self.colors

        plt.ioff()
        plt.clf()

        self.s = st.svg(r"wecs_pmsm_dashboard.svg")

        self.html = widgets.HTML(
            value= self.s.tostring(),
            placeholder='',
            description='',
        )

        # tab_0 = widgets.VBox([widgets.Text(description='Sn')])
        self.sld_nu_w = widgets.FloatSlider(description='Wind (m/s)',min=4, max=20, step=1, value=10)
        # self.tab_1_sld_temp_deg = widgets.FloatSlider(description='temp (deg)',min=10, max=45, step=1, value=25)
        tab_env = widgets.HBox([self.sld_nu_w])

        # tab_2_0 = widgets.RadioButtons(
        #     options=['Manual', 'Power from MPPT', 'Power from speed control'],
        #     description='Mode:',    disabled=False)
        # self.sld_p_s = widgets.FloatSlider(description='p<sub>s</sub>*',min=0, max=1, step=0.1, value=0)
        # self.sld_v_dc = widgets.FloatSlider(description='v<sub>dc</sub>*',min=0.8, max=2.0, step=0.1, value=1.2)
        self.sld_q_s = widgets.FloatSlider(description='q<sub>s</sub>*',min=-1, max=1, step=0.1, value=0)
        # self.sld_i_d = widgets.FloatSlider(description='i<sub>d</sub>*',min=-1, max=1, step=0.1, value=0)
        # self.sld_i_q = widgets.FloatSlider(description='i<sub>q</sub>*',min=-0.1, max=0.1, step=0.1, value=0)

        # tab_2_1 = widgets.VBox([self.sld_p_s,
        #                         ])
        # tab_2 = widgets.HBox([tab_2_0,tab_2_1])

        # self.options_mode = widgets.RadioButtons(
        #     options=['Normal', 'LVRT'],
        #     description='Mode:',    disabled=False)

        tab_vsc_g = widgets.HBox([widgets.VBox([self.sld_q_s])])
        # tab_3 = widgets.HBox([tab_3_0])

        # self.sld_V_g = widgets.FloatSlider(description='V<sub>g</sub>',min=0.2, max=1.2, step=0.1, value=1.0)
        # self.sld_SCR = widgets.FloatSlider(description='SCR',min=1, max=10, step=1, value=10)

        self.sld_p_ext = widgets.FloatSlider(description='p<sub>ext</sub>',min=-1.0, max=0, step=0.1, value=0.0)
        self.sld_beta_ext = widgets.FloatSlider(description='𝛽<sub>ext</sub>',min=0.0, max=60, step=5.0, value=0.0)

        tab_courtailment = widgets.HBox([self.sld_p_ext,self.sld_beta_ext])

        # tab_grid = widgets.HBox([self.sld_V_g,self.sld_SCR])

        self.tab = widgets.Tab()
        self.tab.children = [tab_env,tab_vsc_g,tab_courtailment]

        #tab.set_title(0, 'PV')
        self.tab.set_title(0, 'Enviroment')
        #tab.set_title(2, 'MPPT')
        self.tab.set_title(1, 'VSC-G')
        self.tab.set_title(2, 'Courtailment')
        # self.tab.set_title(2, 'Grid')

        nu_w =self.sld_nu_w.value
        omega_0 = np.clip(nu_w/10,0.4,1.2)
        xy_0 = {
            "V_1": 1.0,
            "theta_1": 0.0,
            "V_2": 1.0,
            "theta_2": 0.0,
            "omega_coi": 1.0,
            "omega_2": 1.0,
            "theta_tr_1": 0.0,
            "omega_t_1": omega_0,
            "omega_r_1": omega_0,
            "xi_beta_1":1,
            "beta_1":0.0,
            "i_sr_1": 1,
            "v_dc_1": 1.5,
            "p_w_mppt_lpf_1":1.2,
            "v_mq_1":1.0,
            "v_tq_ref_1":1.0
        }


        self.model.ini({'K_pdc_1':50,"C_dc_1":0.01,"nu_w_1":nu_w,
                "K_p_beta_1":1,"K_i_beta_1":1.0,"T_beta_1":1.2},xy_0)#'xy_0.json')
        
    def update(self,change):
        
        model = self.model

        nu_w =self.sld_nu_w.value
        Omega_r_max = 1.35
        omega_0 = np.clip(nu_w/10,0.4,Omega_r_max)
        beta_0 = np.clip((nu_w-13)*3,0,90)
        xy_0 = {
            "V_1": 1.0,
            "theta_1": 0.0,
            "V_2": 1.0,
            "theta_2": 0.0,
            "omega_coi": 1.0,
            "omega_2": 1.0,
            "theta_tr_1": 0.0,
            "omega_t_1": omega_0,
            "omega_r_1": omega_0,
            "xi_beta_1":beta_0,
            "beta_1":beta_0,
            "i_sr_1": 1,
            "v_dc_1": 1.5,
            "p_w_mppt_lpf_1":omega_0,
            "v_mq_1":1.0,
            "v_tq_ref_1":1.0
        }

        self.model.ini({'K_pdc_1':50,"C_dc_1":0.01,"nu_w_1":nu_w,'q_s_ref_1':self.sld_q_s.value,
                "K_p_beta_1":1,"K_i_beta_1":1.0,"T_beta_1":1.2,"Omega_r_max_1":Omega_r_max,
                "beta_ext_1":self.sld_beta_ext.value,"p_ref_ext_1":self.sld_p_ext.value},xy_0)#'xy_0.json')

        # s.set_tspan('p_g', f'={model.get_value("p_g_"):0.2f}')
        # s.set_tspan('q_g', f'={model.get_value("q_g_"):0.2f}')
        I_s = (model.get_value("i_si_1")**2 + model.get_value("i_sr_1")**2)**0.5
        self.s.set_tspan('I_s', f'={I_s:4.2f}')
        self.s.set_tspan('i_sd_ref', f'={model.get_value("i_sd_ref_1"):5.2f}')
        self.s.set_tspan('i_sq_ref', f'={model.get_value("i_sq_ref_1"):5.2f}')
        self.s.set_tspan('i_md_ref', f'={0:5.2f}')
        self.s.set_tspan('i_mq_ref', f'={model.get_value("i_mq_ref_1"):5.2f}')
        # self.s.set_tspan('m', f'={model.get_value("m_ref_1"):0.2f}')
        self.s.set_tspan('p_s', f'={model.get_value("p_s_1"):5.2f}')
        self.s.set_tspan('q_s', f'={model.get_value("q_s_1"):5.2f}')
        self.s.set_tspan('p_m', f'={model.get_value("p_m_1"):5.2f}')
        self.s.set_tspan('q_m', f'={model.get_value("q_m_1"):5.2f}')
        self.s.set_tspan('v_dc', f'={model.get_value("v_dc_1"):0.2f}')
        self.s.set_tspan('nu_w', f'={model.get_value("nu_w_1"):3.0f} m/s')
        self.s.set_tspan('beta', f'={model.get_value("beta_1"):4.1f}º')
        self.s.set_tspan('omega_t', f'={model.get_value("omega_t_1"):5.2f}')
        self.s.set_tspan('V_pcc', f'={model.get_value("V_1"):5.2f}')
        self.s.set_tspan('V_g', f'={model.get_value("V_2"):5.2f}')
        # s.set_tspan('beta', f'={np.abs(model.get_value("beta_")):5.2f}')

        self.html.value = self.s.tostring() 


    def show(self):

        # Link the slider value to the text element update function
        self.sld_nu_w.observe(self.update, names='value')
        #self.tab_1_sld_temp_deg.observe(self.update, names='value')
        # self.sld_v_dc.observe(self.update, names='value')
        self.sld_q_s.observe(self.update, names='value')
        self.sld_p_ext.observe(self.update, names='value')
        self.sld_beta_ext.observe(self.update, names='value')
        # self.sld_i_d.observe(self.update, names='value')
        # self.sld_i_q.observe(self.update, names='value')
        # self.sld_V_g.observe(self.update, names='value')
        # self.sld_SCR.observe(self.update, names='value')
        # self.options_mode.observe(self.update, names='value')

        self.update(0)
        self.layout = widgets.VBox([self.html,self.tab])
        display(self.layout)


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym


def pss_kundur(dae,syn_data,name):
    '''
    PSS from Kundur's book1 

    
    '''
    pss_data = syn_data['pss']
    
    omega = sym.Symbol(f"omega_{name}", real=True)   
       
    x_wo  = sym.Symbol(f"x_wo_{name}", real=True)
    x_lead  = sym.Symbol(f"x_lead_{name}", real=True)

    z_wo  = sym.Symbol(f"z_wo_{name}", real=True)
    x_lead  = sym.Symbol(f"x_lead_{name}", real=True)
    
    T_wo = sym.Symbol(f"T_wo_{name}", real=True)  
    T_1 = sym.Symbol(f"T_1_{name}", real=True) 
    T_2 = sym.Symbol(f"T_2_{name}", real=True)
    K_stab = sym.Symbol(f"K_stab_{name}", real=True)
    V_lim = sym.Symbol(f"V_lim_{name}", real=True)
    v_s = sym.Symbol(f"v_pss_{name}", real=True) 
    
    
    u_wo = omega - 1.0
    v_pss_nosat = K_stab*((z_wo - x_lead)*T_1/T_2 + x_lead)
    
    dx_wo =   (u_wo - x_wo)/T_wo  # washout state
    dx_lead =  (z_wo - x_lead)/T_2      # lead compensator state
    
    g_z_wo =  (u_wo - x_wo) - z_wo  
    g_v_s = -v_s + sym.Piecewise((-V_lim,v_pss_nosat<-V_lim),(V_lim,v_pss_nosat>V_lim),(v_pss_nosat,True))  
    
    
    dae['f'] += [dx_wo,dx_lead]
    dae['x'] += [ x_wo, x_lead]
    dae['g'] += [g_z_wo,g_v_s]
    dae['y_ini'] += [  z_wo, v_s]  
    dae['y_run'] += [  z_wo, v_s] 
    dae['params_dict'].update({str(T_wo):pss_data['T_wo']})
    dae['params_dict'].update({str(T_1):pss_data['T_1']})
    dae['params_dict'].update({str(T_2):pss_data['T_2']})
    dae['params_dict'].update({str(K_stab):pss_data['K_stab']})
    dae['params_dict'].update({str(V_lim):0.1})


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym
from pydae.bmapu.vsc_ctrls.ctrl_pq import ctrl_pq
from pydae.bmapu.vsc_ctrls.leon_vsg_ll import leon_vsg_ll
from pydae.bmapu.vsc_ctrls.uvsg import uvsg
from pydae.bmapu.vsc_ctrls.i_vsg_ll import i_vsg_ll

def add_ctrl(dae,name,bus_name,data_dict):
    
    if data_dict['ctrl']['type'] == 'pq':
        ctrl_pq(dae,name,bus_name,data_dict)
    if data_dict['ctrl']['type'] == 'leon_vsg_ll':
        leon_vsg_ll(dae,name,bus_name,data_dict)        
    if data_dict['ctrl']['type'] == 'uvsg':
        uvsg(dae,name,bus_name,data_dict)   
    if data_dict['ctrl']['type'] == 'i_vsg_ll':
        i_vsg_ll(dae,name,bus_name,data_dict)  



import numpy as np
import sympy as sym

def ac_3ph_4w(grid,data):
    '''
    VSC with 3 phase and 4 wire with a PI-VSG with PFR and Q control.
         
    '''
    name = data['bus']


    params_dict  = grid.dae['params_dict']
    f_list = grid.dae['f']
    x_list = grid.dae['x']
    g_list = grid.dae['g'] 
    y_ini_list = grid.dae['y_ini'] 
    u_ini_dict = grid.dae['u_ini_dict']
    y_run_list = grid.dae['y_run'] 
    u_run_dict = grid.dae['u_run_dict']
    h_dict = grid.dae['h_dict']


    # VSC

    ## VSC parameters
    S_n,U_n  = sym.symbols(f'S_n_{name},U_n_{name}', real=True)
    R_s,R_sn,R_ng = sym.symbols(f'R_s_{name},R_sn_{name},R_ng_{name}', real=True)
    X_s,X_sn,X_ng = sym.symbols(f'X_s_{name},X_sn_{name},X_ng_{name}', real=True)
    A_loss = sym.symbols(f'A_loss_{name}',real=True)
    B_loss = sym.symbols(f'B_loss_{name}',real=True)
    C_loss = sym.symbols(f'C_loss_{name}',real=True)

    ## VSC inputs
    v_ta_r, v_ta_i = sym.symbols(f'v_ta_r_{name},v_ta_i_{name}', real=True)
    v_tb_r, v_tb_i = sym.symbols(f'v_tb_r_{name},v_tb_i_{name}', real=True)
    v_tc_r, v_tc_i = sym.symbols(f'v_tc_r_{name},v_tc_i_{name}', real=True)
    v_tn_r, v_tn_i = sym.symbols(f'v_tn_r_{name},v_tn_i_{name}', real=True)


    # algebraic states
    #e_an_i,e_bn_i,e_cn_i,e_ng_i = sym.symbols(f'e_{name}_an_i,e_{name}_bn_i,e_{name}_cn_i,e_{name}_ng_i', real=True)
    #v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'v_{name}_a_r,v_{name}_b_r,v_{name}_c_r,v_{name}_n_r,v_{name}_o_r', real=True)
    #v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'v_{name}_a_i,v_{name}_b_i,v_{name}_c_i,v_{name}_n_i,v_{name}_o_i', real=True)
    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'V_{name}_0_r,V_{name}_1_r,V_{name}_2_r,V_{name}_3_r,v_{name}_o_r', real=True)
    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'V_{name}_0_i,V_{name}_1_i,V_{name}_2_i,V_{name}_3_i,v_{name}_o_i', real=True)
    i_sa_r,i_sb_r,i_sc_r,i_sn_r,i_ng_r = sym.symbols(f'i_vsc_{name}_a_r,i_vsc_{name}_b_r,i_vsc_{name}_c_r,i_vsc_{name}_n_r,i_vsc_{name}_ng_r', real=True)
    i_sa_i,i_sb_i,i_sc_i,i_sn_i,i_ng_i = sym.symbols(f'i_vsc_{name}_a_i,i_vsc_{name}_b_i,i_vsc_{name}_c_i,i_vsc_{name}_n_i,i_vsc_{name}_ng_i', real=True)
    v_ta_r,v_tb_r,v_tc_r,v_tn_r = sym.symbols(f'v_ta_r_{name},v_tb_r_{name},v_tc_r_{name},v_tn_r_{name}', real=True)
    v_ta_i,v_tb_i,v_tc_i,v_tn_i = sym.symbols(f'v_ta_i_{name},v_tb_i_{name},v_tc_i_{name},v_tn_i_{name}', real=True)
    p_dc = sym.Symbol(f'p_dc_{name}', real=True)

    #e_om_r,e_om_i = sym.symbols(f'e_{name}_om_r,e_{name}_om_i', real=True)

    ## VSC impedances
    Z_sa = R_s + 1j*X_s 
    Z_sb = R_s + 1j*X_s
    Z_sc = R_s + 1j*X_s
    Z_sn = R_sn + 1j*X_sn
    Z_ng = R_ng + 1j*X_ng

    ## POI currents
    i_sa = i_sa_r + 1j*i_sa_i
    i_sb = i_sb_r + 1j*i_sb_i
    i_sc = i_sc_r + 1j*i_sc_i
    i_sn = i_sn_r + 1j*i_sn_i

    ## POI voltages
    v_sa = v_sa_r + 1j*v_sa_i
    v_sb = v_sb_r + 1j*v_sb_i
    v_sc = v_sc_r + 1j*v_sc_i
    v_sn = v_sn_r + 1j*v_sn_i
    v_og = v_og_r + 1j*v_og_i

    ## VSC AC side voltages
    v_ta = v_ta_r + 1j*v_ta_i
    v_tb = v_tb_r + 1j*v_tb_i
    v_tc = v_tc_r + 1j*v_tc_i
    v_tn = v_tn_r + 1j*v_tn_i
    
    s_ta = v_ta * sym.conjugate(i_sa)
    s_tb = v_tb * sym.conjugate(i_sb)
    s_tc = v_tc * sym.conjugate(i_sc)
    s_tn = v_tn * sym.conjugate(i_sc)

    p_ac_a = sym.re(s_ta)
    p_ac_b = sym.re(s_tb)
    p_ac_c = sym.re(s_tc)
    p_ac_n = sym.re(s_tn)

    ## VSC RMS currents
    i_rms_a = sym.sqrt(i_sa_r**2+i_sa_i**2+1e-6) 
    i_rms_b = sym.sqrt(i_sb_r**2+i_sb_i**2+1e-6) 
    i_rms_c = sym.sqrt(i_sc_r**2+i_sc_i**2+1e-6) 
    i_rms_n = sym.sqrt(i_sn_r**2+i_sn_i**2+1e-6) 

    ## VSC power losses
    p_loss_a = A_loss*i_rms_a*i_rms_a + B_loss*i_rms_a + C_loss
    p_loss_b = A_loss*i_rms_b*i_rms_b + B_loss*i_rms_b + C_loss
    p_loss_c = A_loss*i_rms_c*i_rms_c + B_loss*i_rms_c + C_loss
    p_loss_n = A_loss*i_rms_n*i_rms_n + B_loss*i_rms_n + C_loss

    p_vsc_loss = p_loss_a + p_loss_b + p_loss_c + p_loss_n
    p_ac = p_ac_a + p_ac_b + p_ac_c + p_ac_n

    eq_i_sa_cplx = v_og + v_ta - i_sa*Z_sa - v_sa   # v_sa = v_sag
    eq_i_sb_cplx = v_og + v_tb - i_sb*Z_sb - v_sb
    eq_i_sc_cplx = v_og + v_tc - i_sc*Z_sc - v_sc
    eq_i_sn_cplx = v_og + v_tn - i_sn*Z_sn - v_sn
    eq_v_og_cplx = i_sa + i_sb + i_sc + i_sn + v_og/Z_ng
    eq_p_dc = -p_dc + p_ac + p_vsc_loss

    grid.dae['g'] += [sym.re(eq_i_sa_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_sb_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_sc_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_sn_cplx)] 
    grid.dae['g'] += [sym.re(eq_v_og_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_sa_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_sb_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_sc_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_sn_cplx)] 
    grid.dae['g'] += [sym.im(eq_v_og_cplx)] 
    grid.dae['g'] += [eq_p_dc] 

    grid.dae['y_ini'] += [i_sa_r,i_sb_r,i_sc_r,i_sn_r,v_og_r]
    grid.dae['y_ini'] += [i_sa_i,i_sb_i,i_sc_i,i_sn_i,v_og_i]
    grid.dae['y_ini'] += [p_dc]
    grid.dae['y_run'] += [i_sa_r,i_sb_r,i_sc_r,i_sn_r,v_og_r]
    grid.dae['y_run'] += [i_sa_i,i_sb_i,i_sc_i,i_sn_i,v_og_i]
    grid.dae['y_run'] += [p_dc]

    for ph in ['a','b','c','n']:
        i_s_r = sym.Symbol(f'i_vsc_{name}_{ph}_r', real=True)
        i_s_i = sym.Symbol(f'i_vsc_{name}_{ph}_i', real=True)  
        idx_r,idx_i = grid.node2idx(name,ph)
        grid.dae['g'] [idx_r] += -i_s_r
        grid.dae['g'] [idx_i] += -i_s_i
        i_s = i_s_r + 1j*i_s_i
        i_s_m = np.abs(i_s)
        grid.dae['h_dict'].update({f'i_vsc_{name}_{ph}_m':i_s_m})

    ## outputs
    v_sabc = sym.Matrix([[v_sa],[v_sb],[v_sc]])
    i_sabc = sym.Matrix([[i_sa],[i_sb],[i_sc]])
    n2a = {0:'a',1:'b',2:'c'}
    for ph in [0,1,2]:
        s = v_sabc[ph]*sym.conjugate(i_sabc[ph])
        p = sym.re(s)
        q = sym.im(s)
        grid.dae['h_dict'].update({f'p_vsc_{name}_{n2a[ph]}':p,f'q_vsc_{name}_{n2a[ph]}':q})

    ## parameters default values
    grid.dae['params_dict'].update({f'X_s_{name}':data['X'],f'R_s_{name}':data['R']})
    grid.dae['params_dict'].update({f'X_sn_{name}':data['X_n'],f'R_sn_{name}':data['R_n']})
    grid.dae['params_dict'].update({f'X_ng_{name}':data['X_ng'],f'R_ng_{name}':data['R_ng']})

    V_n_N = data['U_n']/np.sqrt(3)
    phi_N = 0.0
    if 'phi' in data: phi_N = data['phi']

    v_ta_N = V_n_N*np.exp(1j*phi_N)
    v_tb_N = V_n_N*np.exp(1j*(phi_N - np.pi*2.0/3.0))
    v_tc_N = V_n_N*np.exp(1j*(phi_N - np.pi*4.0/3.0))
    v_tn_N = 0.0

    # the following are inputs in this models, however they should be states in the controllers, 
    # for reducing complexity at the controllers the initaializations are performed here:
    grid.dae['xy_0_dict'].update({str(v_ta_r):v_ta_N.real, str(v_ta_i):v_ta_N.imag})
    grid.dae['xy_0_dict'].update({str(v_tb_r):v_tb_N.real, str(v_tb_i):v_tb_N.imag})
    grid.dae['xy_0_dict'].update({str(v_tc_r):v_tc_N.real, str(v_tc_i):v_tc_N.imag})
    grid.dae['xy_0_dict'].update({str(v_tn_r):v_tn_N.real, str(v_tn_i):v_tn_N.imag})

    S_n_N = data['S_n']
    U_n_N = data['U_n']
    V_n_N = U_n_N/np.sqrt(3)
    phi_N = 0.0
    if 'phi' in data: phi_N = data['phi']

    v_ta_N = V_n_N*np.exp(1j*phi_N)
    v_tb_N = V_n_N*np.exp(1j*(phi_N - np.pi*2.0/3.0))
    v_tc_N = V_n_N*np.exp(1j*(phi_N - np.pi*4.0/3.0))
    v_tn_N = 0.0

    I_n = S_n_N/(np.sqrt(3)*U_n_N)
    P_0 = 0.01*S_n_N
    C_loss_N = P_0/3
    P_cc = 0.01*S_n_N
    A_loss_N = P_cc/(I_n**2)/3
    B_loss_N = 1.0

    if 'A_loss' in data:
        A_loss_N = data['A_loss']
        B_loss_N = data['B_loss']
        C_loss_N = data['C_loss']

    params_dict.update({str(A_loss):A_loss_N})
    params_dict.update({str(B_loss):B_loss_N})
    params_dict.update({str(C_loss):C_loss_N})

    grid.dae['u_ini_dict'].update({str(v_ta_r):v_ta_N.real, str(v_ta_i):v_ta_N.imag})
    grid.dae['u_ini_dict'].update({str(v_tb_r):v_tb_N.real, str(v_tb_i):v_tb_N.imag})
    grid.dae['u_ini_dict'].update({str(v_tc_r):v_tc_N.real, str(v_tc_i):v_tc_N.imag})
    grid.dae['u_ini_dict'].update({str(v_tn_r):v_tn_N.real, str(v_tn_i):v_tn_N.imag})
    grid.dae['u_run_dict'].update({str(v_ta_r):v_ta_N.real, str(v_ta_i):v_ta_N.imag})
    grid.dae['u_run_dict'].update({str(v_tb_r):v_tb_N.real, str(v_tb_i):v_tb_N.imag})
    grid.dae['u_run_dict'].update({str(v_tc_r):v_tc_N.real, str(v_tc_i):v_tc_N.imag})
    grid.dae['u_run_dict'].update({str(v_tn_r):v_tn_N.real, str(v_tn_i):v_tn_N.imag})



def test_ib():
    '''
    test grid former connected to infinite bus
    '''
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db
    import pytest

    grid = urisi('ac_3ph_4w_gfpizv_ib.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    model = temp.model()    
    p_c = 0.5
    q_ref = 0.1
    model.ini({'p_c_A1':p_c,'q_ref_A1':q_ref},'xy_0.json')
    
    model.report_u()
    model.report_x()
    model.report_y()
    model.report_z()

    S_n = model.get_value('S_n_A1')
    assert model.get_value('omega_A1')  == pytest.approx(1)
    assert model.get_value('omega_coi') == pytest.approx(1)
    assert model.get_value('p_vsc_A1_a') == pytest.approx(p_c*S_n/3, rel=0.05)
    assert model.get_value('p_vsc_A1_b') == pytest.approx(p_c*S_n/3, rel=0.05)
    assert model.get_value('p_vsc_A1_c') == pytest.approx(p_c*S_n/3, rel=0.05)
    assert model.get_value('p_A2') == pytest.approx(-p_c*S_n, rel=0.05)
    assert model.get_value('q_A2') == pytest.approx(-q_ref*S_n, rel=0.05)

def test_iso():
    '''
    test isolated grid former feeding a load
    '''
    import numpy as np
    import sympy as sym
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db
    import pytest

    grid = urisi('ac_3ph_4w_gfpizv_iso.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    model = temp.model()
    p_load_ph =  50e3
    q_load_ph =  20e3
    model.ini({'p_load_A2_a':p_load_ph,'q_load_A2_a':q_load_ph,
               'p_load_A2_b':p_load_ph,'q_load_A2_b':q_load_ph,
               'p_load_A2_c':p_load_ph,'q_load_A2_c':q_load_ph,},'xy_0.json')

    model.report_x()  
    model.report_y()
    model.report_z()

    assert model.get_value('omega_A1')  == 1.0
    assert model.get_value('omega_coi') == 1.0
    assert model.get_value('p_vsc_A1_a') == pytest.approx(p_load_ph, rel=0.05)
    assert model.get_value('p_vsc_A1_b') == pytest.approx(p_load_ph, rel=0.05)
    assert model.get_value('p_vsc_A1_c') == pytest.approx(p_load_ph, rel=0.05)
    assert model.get_value('q_vsc_A1_a') == pytest.approx(q_load_ph, rel=0.05)
    assert model.get_value('q_vsc_A1_b') == pytest.approx(q_load_ph, rel=0.05)
    assert model.get_value('q_vsc_A1_c') == pytest.approx(q_load_ph, rel=0.05)


if __name__ == '__main__':

    #development()
    test_ib()


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def ctrl_pq(grid,name,bus_name,data_dict):
    '''

    parameters
    ----------

    S_n: nominal power in VA
    F_n: nominal frequency in Hz
    X_v: coupling reactance in pu (base machine S_n)
    R_v: coupling resistance in pu (base machine S_n)
    K_delta: if K_delta>0.0 current generator is converted to reference machine 
    K_alpha: alpha gain to obtain Domega integral 

    inputs
    ------

    alpha: RoCoF in pu if K_alpha = 1.0
    omega_ref: frequency in pu
    v_ref: internal voltage reference

    example
    -------

    "genapes": [{"S_n":1e9,"F_n":50.0,"X_v":0.001,"R_v":0.0,"K_delta":0.001,"K_alpha":1e-6}]

    S_n = sym.Symbol(f"S_n_{name}", real=True)
    F_n = sym.Symbol(f"F_n_{name}", real=True)            
    X_v = sym.Symbol(f"X_v_{name}", real=True)
    R_v = sym.Symbol(f"R_v_{name}", real=True)
    K_delta = sym.Symbol(f"K_delta_{name}", real=True)
    K_alpha = sym.Symbol(f"K_alpha_{name}", real=True)

    
    '''

    sin = sym.sin
    cos = sym.cos

    ctrl_data = data_dict['ctrl']

    # inputs
    V_s = sym.Symbol(f"V_{bus_name}", real=True)
    theta_s = sym.Symbol(f"theta_{bus_name}", real=True)
    omega_coi = sym.Symbol("omega_coi", real=True)   
    v_dc = sym.Symbol(f"v_dc_{name}", real=True)   
    
    # dynamic states
    delta = sym.Symbol(f"delta_{name}", real=True)

    # algebraic states
    i_sd_ref = sym.Symbol(f"i_sd_ref_{name}", real=True)
    i_sq_ref = sym.Symbol(f"i_sq_ref_{name}", real=True)   
    p_s_ref = sym.Symbol(f"p_s_ref_{name}", real=True)
    q_s_ref = sym.Symbol(f"q_s_ref_{name}", real=True)      
    v_td_ref = sym.Symbol(f"v_td_ref_{name}", real=True)
    v_tq_ref = sym.Symbol(f"v_tq_ref_{name}", real=True)    
    m = sym.Symbol(f"m_{name}", real=True) 
    theta_t = sym.Symbol(f"theta_t_{name}", real=True) 

    # parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    F_n = sym.Symbol(f"F_n_{name}", real=True)            
    X_s = sym.Symbol(f"X_s_{name}", real=True)
    R_s = sym.Symbol(f"R_s_{name}", real=True)
    
    params_list = []
    
    # auxiliar
    delta = theta_s
    v_sD = V_s*sin(theta_s)  # v_si   e^(-j)
    v_sQ = V_s*cos(theta_s)  # v_sr
    v_sd = v_sD * cos(delta) - v_sQ * sin(delta)   
    v_sq = v_sD * sin(delta) + v_sQ * cos(delta)
    
    Omega_b = 2*np.pi*F_n
    omega_s = omega_coi
    v_tD_ref = v_td_ref * cos(delta) + v_tq_ref * sin(delta)   
    v_tQ_ref = v_td_ref * sin(delta) - v_tq_ref * cos(delta)    
    v_ti_ref =  v_tD_ref
    v_tr_ref =  v_tQ_ref   
    m_ref = sym.sqrt(v_tr_ref**2 + v_ti_ref**2)/v_dc
    theta_t_ref = sym.atan2(v_ti_ref,-v_tr_ref) 

    # dynamic equations            


    # algebraic equations   
    g_i_sd_ref  = i_sd_ref*v_sd + i_sq_ref*v_sq - p_s_ref  
    g_i_sq_ref  = i_sd_ref*v_sq - i_sq_ref*v_sd - q_s_ref
    g_v_td_ref  = v_td_ref - R_s*i_sd_ref + X_s*i_sq_ref - v_sd  
    g_v_tq_ref  = v_tq_ref - R_s*i_sq_ref - X_s*i_sd_ref - v_sq 
    g_m  = m-m_ref
    g_theta_t  = theta_t-theta_t_ref
   
    # dae 
    f_vsg = []
    x_vsg = []
    g_vsg = [g_i_sd_ref,g_i_sq_ref,g_v_td_ref,g_v_tq_ref,g_m,g_theta_t]
    y_vsg = [  i_sd_ref,  i_sq_ref,  v_td_ref,  v_tq_ref,  m,  theta_t]
    
    grid.dae['f'] += f_vsg
    grid.dae['x'] += x_vsg
    grid.dae['g'] += g_vsg
    grid.dae['y_ini'] += y_vsg  
    grid.dae['y_run'] += y_vsg  
    
    v_sq_ref_0 = 1.0
    p_s_ref_0 = ctrl_data['p_s_ref']
    q_s_ref_0 = ctrl_data['q_s_ref']
    i_sd_ref_0 = p_s_ref_0/v_sq_ref_0
    i_sq_ref_0 = q_s_ref_0/v_sq_ref_0

    grid.dae['u_ini_dict'].update({f'{str(p_s_ref)}':p_s_ref_0})
    grid.dae['u_run_dict'].update({f'{str(p_s_ref)}':p_s_ref_0})

    grid.dae['u_ini_dict'].update({f'{str(q_s_ref)}':q_s_ref_0})
    grid.dae['u_run_dict'].update({f'{str(q_s_ref)}':q_s_ref_0})

    grid.dae['u_ini_dict'].update({f'{str(i_sd_ref)}':i_sd_ref_0})
    grid.dae['u_run_dict'].update({f'{str(i_sd_ref)}':i_sd_ref_0})

    grid.dae['u_ini_dict'].update({f'{str(i_sq_ref)}':i_sq_ref_0})
    grid.dae['u_run_dict'].update({f'{str(i_sq_ref)}':i_sq_ref_0})

      
    # outputs
    grid.dae['h_dict'].update({f"m_ref_{name}":m_ref})
    grid.dae['h_dict'].update({f"theta_t_{name}":theta_t})
    
    for item in params_list:       
        grid.dae['params_dict'].update({f"{item}_{name}":ctrl_data[item]}) 

    return m,theta_t



import numpy as np
import sympy as sym

def breaker(grid,data):
    '''
    VSC with 3 phase and 4 wire with a PI-VSG with PFR and Q control.
         
    '''
    name1 = data['bus_1']
    name2 = data['bus_2']

    params_dict  = grid.dae['params_dict']
    f_list = grid.dae['f']
    x_list = grid.dae['x']
    g_list = grid.dae['g'] 
    y_ini_list = grid.dae['y_ini'] 
    u_ini_dict = grid.dae['u_ini_dict']
    y_run_list = grid.dae['y_run'] 
    u_run_dict = grid.dae['u_run_dict']
    h_dict = grid.dae['h_dict']


    # VSC

    ## VSC parameters
    S_n,U_n  = sym.symbols(f'S_n_{name1},U_n_{name1}', real=True)
    R,X = sym.symbols(f'R_{name1},X_{name1}', real=True)

    ## inputs
    u_brk = sym.symbols(f'u_brk_{name1}', real=True)

    # algebraic states
    v_1a_r,v_1b_r,v_1c_r,v_1n_r = sym.symbols(f'V_{name1}_0_r,V_{name1}_1_r,V_{name1}_2_r,V_{name1}_3_r', real=True)
    v_1a_i,v_1b_i,v_1c_i,v_1n_i = sym.symbols(f'V_{name1}_0_i,V_{name1}_1_i,V_{name1}_2_i,V_{name1}_3_i', real=True)
    i_1a_r,i_1b_r,i_1c_r,i_1n_r = sym.symbols(f'i_brk_{name1}_a_r,i_brk_{name1}_b_r,i_brk_{name1}_c_r,i_brk_{name1}_n_r', real=True)
    i_1a_i,i_1b_i,i_1c_i,i_1n_i = sym.symbols(f'i_brk_{name1}_a_i,i_brk_{name1}_b_i,i_brk_{name1}_c_i,i_brk_{name1}_n_i', real=True)
    v_2a_r,v_2b_r,v_2c_r,v_2n_r = sym.symbols(f'V_{name2}_0_r,V_{name2}_1_r,V_{name2}_2_r,V_{name2}_3_r', real=True)
    v_2a_i,v_2b_i,v_2c_i,v_2n_i = sym.symbols(f'V_{name2}_0_i,V_{name2}_1_i,V_{name2}_2_i,V_{name2}_3_i', real=True)
    i_2a_r,i_2b_r,i_2c_r,i_2n_r = sym.symbols(f'i_brk_{name2}_a_r,i_brk_{name2}_b_r,i_brk_{name2}_c_r,i_brk_{name2}_n_r', real=True)
    i_2a_i,i_2b_i,i_2c_i,i_2n_i = sym.symbols(f'i_brk_{name2}_a_i,i_brk_{name2}_b_i,i_brk_{name2}_c_i,i_brk_{name2}_n_i', real=True)

    #e_om_r,e_om_i = sym.symbols(f'e_{name}_om_r,e_{name}_om_i', real=True)
    ## VSC impedances
    Z = R + 1j*X

    ## POI currents
    i_1a = i_1a_r + 1j*i_1a_i
    i_1b = i_1b_r + 1j*i_1b_i
    i_1c = i_1c_r + 1j*i_1c_i
    i_1n = i_1n_r + 1j*i_1n_i

    ## POI currents
    i_2a = i_2a_r + 1j*i_2a_i
    i_2b = i_2b_r + 1j*i_2b_i
    i_2c = i_2c_r + 1j*i_2c_i
    i_2n = i_2n_r + 1j*i_2n_i

    ## VSC AC side voltages
    v_1a = v_1a_r + 1j*v_1a_i
    v_1b = v_1b_r + 1j*v_1b_i
    v_1c = v_1c_r + 1j*v_1c_i
    v_1n = v_1n_r + 1j*v_1n_i
    
    ## VSC AC side voltages
    v_2a = v_2a_r + 1j*v_2a_i
    v_2b = v_2b_r + 1j*v_2b_i
    v_2c = v_2c_r + 1j*v_2c_i
    v_2n = v_2n_r + 1j*v_2n_i

    eq_i_1a_cplx = v_1a - i_1a*Z - v_2a   # v_sa = v_sag
    eq_i_1b_cplx = v_1b - i_1b*Z - v_2b
    eq_i_1c_cplx = v_1c - i_1c*Z - v_2c
    eq_i_1n_cplx = v_1n - i_1n*Z - v_2n

    grid.dae['g'] += [sym.re(eq_i_1a_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_1b_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_1c_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_1n_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_1a_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_1b_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_1c_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_1n_cplx)] 

    grid.dae['y_ini'] += [i_1a_r,i_1b_r,i_1c_r,i_1n_r]
    grid.dae['y_ini'] += [i_1a_i,i_1b_i,i_1c_i,i_1n_i]
    grid.dae['y_run'] += [i_1a_r,i_1b_r,i_1c_r,i_1n_r]
    grid.dae['y_run'] += [i_1a_i,i_1b_i,i_1c_i,i_1n_i]

    for ph in ['a','b','c','n']:  
        i_brk_r = sym.Symbol(f'i_brk_{name1}_{ph}_r', real=True)
        i_brk_i = sym.Symbol(f'i_brk_{name1}_{ph}_i', real=True)  
        idx_r,idx_i = grid.node2idx(name1,ph)
        grid.dae['g'] [idx_r] += -i_brk_r*u_brk
        grid.dae['g'] [idx_i] += -i_brk_i*u_brk
        i_brk = i_brk_r + 1j*i_brk_i
        i_brk_m = np.abs(i_brk)
        grid.dae['h_dict'].update({f'i_brk_{name1}_{ph}_m':i_brk_m})

    for ph in ['a','b','c','n']:
        i_brk_r = sym.Symbol(f'i_brk_{name1}_{ph}_r', real=True)
        i_brk_i = sym.Symbol(f'i_brk_{name1}_{ph}_i', real=True)  
        idx_r,idx_i = grid.node2idx(name2,ph)
        grid.dae['g'] [idx_r] +=  i_brk_r*u_brk
        grid.dae['g'] [idx_i] +=  i_brk_i*u_brk
        i_brk = i_brk_r + 1j*i_brk_i
        i_brk_m = np.abs(i_brk)
        grid.dae['h_dict'].update({f'i_brk_{name2}_{ph}_m':i_brk_m})

    grid.dae['u_ini_dict'].update({f'u_brk_{name1}':1.0})
    grid.dae['u_run_dict'].update({f'u_brk_{name1}':1.0})

    grid.dae['params_dict'].update({f'R_{name1}':1e-4})
    grid.dae['params_dict'].update({f'X_{name1}':1e-4})

def test_ib_2src():
    '''
    test grid former connected to infinite bus
    '''
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db
    import pytest

    grid = urisi('breaker_2src.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    model = temp.model()    
    model.ini({
                'p_load_A2_a':100e3,
                'q_load_A2_a':10e3,
                'p_load_A2_b':100e3,
                'q_load_A2_b':10e3,
                'p_load_A2_c':100e3,
                'q_load_A2_c':10e3,
                'u_brk_A2':0.0
                },'xy_0.json')
    
    model.report_u()
    model.report_x()
    model.report_y()
    model.report_z()

    # S_n = model.get_value('S_n_A1')
    # assert model.get_value('omega_A1')  == pytest.approx(1)
    # assert model.get_value('omega_coi') == pytest.approx(1)
    # assert model.get_value('p_vsc_A1_a') == pytest.approx(p_c*S_n/3, rel=0.05)
    # assert model.get_value('p_vsc_A1_b') == pytest.approx(p_c*S_n/3, rel=0.05)
    # assert model.get_value('p_vsc_A1_c') == pytest.approx(p_c*S_n/3, rel=0.05)
    # assert model.get_value('p_A2') == pytest.approx(-p_c*S_n, rel=0.05)
    # assert model.get_value('q_A2') == pytest.approx(-q_ref*S_n, rel=0.05)


if __name__ == '__main__':

    #development()
    test_ib_2src()


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def vsc_pq(grid,name,bus_name,data_dict):
    """
    # auxiliar
    
    """

    p_in = sym.Symbol(f"p_in_{name}", real=True)
    Dp_r = sym.Symbol(f"Dp_r_{name}", real=True)
    Dq_r = sym.Symbol(f"Dq_r_{name}", real=True)

        
    # dynamic states
    p_out = sym.Symbol(f"p_out_{name}", real=True)
    q_out = sym.Symbol(f"q_out_{name}", real=True)

    # algebraic states


    # parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    
    # auxiliar
    
                
    # dynamic equations            


    # algebraic equations   
    p_out_sat = sym.Piecewise((0.0,p_in + Dp_r<0.0),(p_in,p_in + Dp_r>p_in),(p_in + Dp_r,True))         
    q_out_max = (1**2 - p_out_sat**2)**0.5
    q_out_sat = sym.Piecewise((-q_out_max,Dq_r<-q_out_max),(q_out_max,Dq_r>q_out_max),(Dq_r,True))     
    g_p_out = -p_out + p_out_sat
    g_q_out = -q_out + q_out_sat

    # dae 
    grid.dae['f'] += []
    grid.dae['x'] += []
    grid.dae['g'] += [g_p_out,g_q_out]
    grid.dae['y_ini'] += [p_out,q_out]  
    grid.dae['y_run'] += [p_out,q_out]  
    
    grid.dae['u_ini_dict'].update({f'{p_in}':data_dict['p_in']})
    grid.dae['u_run_dict'].update({f'{p_in}':data_dict['p_in']})
    grid.dae['u_ini_dict'].update({f'{Dp_r}':0.0})
    grid.dae['u_run_dict'].update({f'{Dp_r}':0.0})
    grid.dae['u_ini_dict'].update({f'{Dq_r}':0.0})
    grid.dae['u_run_dict'].update({f'{Dq_r}':0.0})            
               
    # outputs
    grid.dae['h_dict'].update({f"{p_in}":p_in})
    grid.dae['h_dict'].update({f"{Dp_r}":Dp_r})
    grid.dae['h_dict'].update({f"{Dq_r}":Dq_r})
    
    # parameters            
    grid.dae['params_dict'].update({f"{S_n}":data_dict['S_n']}) 

    p_W   = p_out*S_n
    q_var = q_out*S_n

    return p_W,q_var


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym
from pydae.bmapu.vscs.vsc_pq import vsc_pq
from pydae.bmapu.vscs.vsc_l  import vsc_l
from pydae.bmapu.vscs.vsc_lcl_uc  import vsc_lcl_uc
from pydae.bmapu.vscs.bess_pq  import bess_pq

from pydae.bmapu.vsc_ctrls.vsc_ctrls import add_ctrl
from pydae.bmapu.vsc_models.vsc_models import add_model

from pydae.bmapu.vscs.vsc_acgf_dcp import vsc_acgf_dcp
from pydae.bmapu.vscs.vsc_acq_dcgf import vsc_acq_dcgf

def add_vscs(grid):

    buses = grid.data['buses']
    buses_list = [bus['name'] for bus in buses]

    for item in grid.data['vscs']:

        data_dict = item
        
        if 'bus' in data_dict:   # single bus VSC
            bus_name = item['bus']
            
            if 'name' in item:
                name = item['name']
            else:
                name = bus_name
                
            for gen_id in range(100):
                if name not in grid.generators_id_list:
                    grid.generators_id_list += [name]
                    break
                else:
                    name = name + f'_{gen_id}'
                    
            item['name'] = name

            if item['type'] == 'vsc_pq':                    
                p_W, q_var = vsc_pq(grid,name,bus_name,data_dict)
            if item['type'] == 'vsc_l':                    
                p_W, q_var = vsc_l(grid,name,bus_name,data_dict)
            if item['type'] == 'vsc_lcl_uc':                    
                p_W, q_var = vsc_lcl_uc(grid,name,bus_name,data_dict)
            if item['type'] == 'bess_pq':                    
                p_W, q_var = bess_pq(grid,name,bus_name,data_dict)




            # grid power injection
            idx_bus = buses_list.index(bus_name) # get the number of the bus where the syn is connected
            if not 'idx_powers' in buses[idx_bus]: buses[idx_bus].update({'idx_powers':0})
            buses[idx_bus]['idx_powers'] += 1

            S_base = sym.Symbol('S_base', real = True)
            grid.dae['g'][idx_bus*2]   += -p_W/S_base
            grid.dae['g'][idx_bus*2+1] += -q_var/S_base


            ## Add controlers and models
            m = f'm_{name}'
            theta_t = f'theta_t_{name}'
            
            if 'ctrl' in item:
                add_ctrl(grid,name,bus_name,data_dict)
                grid.dae['u_ini_dict'].pop(str(m))
                grid.dae['u_run_dict'].pop(str(m))
                grid.dae['u_ini_dict'].pop(str(theta_t))
                grid.dae['u_run_dict'].pop(str(theta_t))
                grid.dae['xy_0_dict'].update({str(m):0.8})
                grid.dae['xy_0_dict'].update({str(theta_t):0.0})


            if 'models' in item:
                add_model(grid,name,bus_name,data_dict)
                grid.dae['u_ini_dict'].pop(str(v_f))
                grid.dae['u_run_dict'].pop(str(v_f))
                grid.dae['xy_0_dict'].update({str(v_f):1.5})

        if 'bus_ac' in data_dict:  # more than one bus VSC

            bus_dc = item['bus_dc']
            bus_ac = item['bus_ac']
            
            if 'name' in item:
                name = item['name']
            else:
                name = f'{bus_dc}_{bus_ac}'
                    
            item['name'] = name

            if item['type'] == 'vsc_acgf_dcp':    # AC side Grid-Former               
                p_W_j, q_var_j,p_W_k, q_var_k = vsc_acgf_dcp(grid,name,bus_dc,bus_ac,data_dict)
            if item['type'] == 'vsc_acq_dcgf':    # DC side Grid-Former               
                p_W_j, q_var_j,p_W_k, q_var_k = vsc_acq_dcgf(grid,name,bus_dc,bus_ac,data_dict)

            # grid power injection
            idx_bus_dc = buses_list.index(bus_dc) # get the number of the bus j
            S_base = sym.Symbol('S_base', real = True)
            grid.dae['g'][idx_bus_dc*2]   += -p_W_j/S_base
            grid.dae['g'][idx_bus_dc*2+1] += -q_var_j/S_base

            idx_bus_ac = buses_list.index(bus_ac) # get the number of the bus k
            S_base = sym.Symbol('S_base', real = True)
            grid.dae['g'][idx_bus_ac*2]   += -p_W_k/S_base
            grid.dae['g'][idx_bus_ac*2+1] += -q_var_k/S_base